@@ -14,7 +14,8 @@
 #include <uim/uim-helper.h>
 #include <uim/gettext.h>
 
-#define NR_CANDIDATES 10
+#define NR_CANDIDATES 20
+#define DEFAULT_SEPARATOR_STR "|"
 
 static int im_uim_fd = -1;
 static int read_tag;
@@ -33,6 +34,7 @@ struct candidate_window {
   int candidate_index;
   int layout_begin;
   int is_active;
+  int display_limit;
   int select_count;
   gboolean left;
   GdkRectangle cursor;
@@ -47,7 +49,6 @@ typedef struct _IMUIMContext {
   int nr_psegs;
   struct preedit_segment *pseg;
   /**/
-  GtkWidget *stat_win;
   GtkWidget *menu;
   GdkWindow *win;
   GdkWindow *toplevel;
@@ -80,6 +81,8 @@ static void update_candidate(IMUIMContext *uic);
 static void
 im_uim_parse_helper_str(char *str);
 
+static gboolean get_user_defined_color(PangoColor *color, const gchar *uim_symbol);
+
 static const GTypeInfo class_info = {
   sizeof(IMContextUIMClass),
   (GBaseInitFunc) NULL,
@@ -122,7 +125,7 @@ clear_cb(void *ptr)
     free(uic->pseg[i].str);
   }
   free(uic->pseg);
-  uic->pseg = 0;
+  uic->pseg = NULL;
   uic->nr_psegs = 0;
 }
 
@@ -132,7 +135,7 @@ pushback_cb(void *ptr, int attr, char *str)
   IMUIMContext *uic = ptr;
   g_return_if_fail(str);
   
-  if(!strcmp(str,"") && attr != UPeAttr_Cursor)
+  if(!strcmp(str,"") && !(attr & (UPeAttr_Cursor | UPeAttr_Separator)))
     return;
 
   uic->pseg = realloc(uic->pseg,
@@ -169,18 +172,6 @@ convert_keyval(int key)
   case GDK_Home: return UKey_Home;
   case GDK_End: return UKey_End;
   case GDK_Zenkaku_Hankaku: return UKey_Zenkaku_Hankaku;
-  case GDK_F1: return UKey_F1;
-  case GDK_F2: return UKey_F2;
-  case GDK_F3: return UKey_F3;
-  case GDK_F4: return UKey_F4;
-  case GDK_F5: return UKey_F5;
-  case GDK_F6: return UKey_F6;
-  case GDK_F7: return UKey_F7;
-  case GDK_F8: return UKey_F8;
-  case GDK_F9: return UKey_F9;
-  case GDK_F10: return UKey_F10;
-  case GDK_F11: return UKey_F11;
-  case GDK_F12: return UKey_F12;
   case GDK_Multi_key: return UKey_Multi_key;
   case GDK_Mode_switch: return UKey_Mode_switch;
   case GDK_Henkan_Mode: return UKey_Henkan_Mode;
@@ -195,6 +186,9 @@ convert_keyval(int key)
   case GDK_Meta_R: return UKey_Meta_key;
   }
 
+  if(key >= GDK_F1 && key <= GDK_F35) {
+    return key - GDK_F1 + UKey_F1;
+  }
   if(key >= GDK_KP_0 && key <= GDK_KP_9) {
     return key - GDK_KP_0 + UKey_0;
   }
@@ -232,36 +226,84 @@ filter_keypress(GtkIMContext *ic,
   return gtk_im_context_filter_keypress(uic->slave, key);
 }
 
+static gboolean
+get_user_defined_color(PangoColor *color, const gchar *uim_symbol) {
+  gboolean parsed = FALSE;
+  gchar *literal = uim_symbol_value_str(uim_symbol);
+  if(literal != NULL && literal[0] != '\0') {
+    parsed = pango_color_parse(color, literal);
+  }
+  g_free(literal);
+  return parsed;
+}
+
 static char *
 get_preedit_segment(struct preedit_segment *ps,
 		    PangoAttrList *attrs,
 		    char *str)
 {
   PangoAttribute *attr;
+  const gchar *segment_str = ps->str;
+  if ((ps->attr & UPeAttr_Separator) && !strcmp(segment_str, "")) {
+    segment_str = DEFAULT_SEPARATOR_STR;
+  }
   if (attrs) {
+    PangoColor color;
     int begin, end;
     begin = strlen(str);
-    end = begin + strlen(ps->str);
+    end = begin + strlen(segment_str);
     if (ps->attr & UPeAttr_UnderLine) {
       attr = pango_attr_underline_new(PANGO_UNDERLINE_SINGLE);
       attr->start_index = begin;
       attr->end_index = end;
       pango_attr_list_change(attrs, attr);
     }
-    if (ps->attr & UPeAttr_Reverse) {
-        attr = pango_attr_foreground_new (0xffff, 0xffff, 0xffff);
-        attr->start_index = begin;
-        attr->end_index = end;
-        pango_attr_list_change (attrs, attr);
+    if (ps->attr & UPeAttr_Separator) {
+      const gchar *separator_fg_symbol, *separator_bg_symbol;
+      if (ps->attr & UPeAttr_Reverse) {
+	separator_fg_symbol = "reversed-separator-foreground";
+	separator_bg_symbol = "reversed-separator-background";
+      } else {
+	separator_fg_symbol = "separator-foreground";
+	separator_bg_symbol = "separator-background";
+      }
+      if (get_user_defined_color(&color, separator_fg_symbol))
+      {
+	attr = pango_attr_foreground_new (color.red, color.green, color.blue);
+	attr->start_index = begin;
+	attr->end_index = end;
+	pango_attr_list_change (attrs, attr);
+      }
 
-        attr = pango_attr_background_new (0, 0, 0);
-        attr->start_index = begin;
-        attr->end_index = end;
-        pango_attr_list_change (attrs, attr);
+      if (get_user_defined_color(&color, separator_bg_symbol))
+      {
+	attr = pango_attr_background_new (color.red, color.green, color.blue);
+	attr->start_index = begin;
+	attr->end_index = end;
+	pango_attr_list_change (attrs, attr);
+      }
+    } else if (ps->attr & UPeAttr_Reverse) {
+      if (get_user_defined_color(&color, "reversed-preedit-foreground")
+	  || pango_color_parse(&color, "#fff"))
+      {
+	attr = pango_attr_foreground_new (color.red, color.green, color.blue);
+	attr->start_index = begin;
+	attr->end_index = end;
+	pango_attr_list_change (attrs, attr);
+      }
+
+      if (get_user_defined_color(&color, "reversed-preedit-background")
+	  || pango_color_parse(&color, "#000"))
+      {
+	attr = pango_attr_background_new (color.red, color.green, color.blue);
+	attr->start_index = begin;
+	attr->end_index = end;
+	pango_attr_list_change (attrs, attr);
+      }
     }
   }
-  str = (char *)realloc(str, strlen(str) + strlen(ps->str)+1);
-  strcat(str, ps->str);
+  str = (char *)realloc(str, strlen(str) + strlen(segment_str)+1);
+  strcat(str, segment_str);
   return str;
 }
 
@@ -305,7 +347,7 @@ im_uim_set_cursor_location(GtkIMContext *ic,
 {
   IMUIMContext *uic = IM_UIM_CONTEXT(ic);
 
-  if(uic->cwin.left &&  uic->nr_psegs > 0){
+  if(uic->cwin.left &&  uic->nr_psegs > 1){
     return;
   }
 
@@ -328,20 +370,6 @@ im_uim_commit_cb(GtkIMContext *ic,
   g_signal_emit_by_name(is, "commit", str);
 }
 
-static void
-update_status_window(IMUIMContext *uic)
-{
-  if (uic->win && uic->stat_win) {
-    gint x,y,w,h,d;
-    gdk_window_get_geometry(uic->win, &x, &y,&w,&h,&d);
-    gdk_window_get_origin(uic->win, &x, &y);
-    gtk_window_move(GTK_WINDOW(uic->stat_win), x+w , y+h);
-  }
-  if(uic->cwin.top_win && uic->cwin.is_active){
-    gtk_widget_show(uic->cwin.top_win);
-  }
-}
-
 static void
 check_helper_connection()
 {
@@ -368,9 +396,6 @@ focus_in(GtkIMContext *ic)
 
   for (cc = context_list.next; cc != &context_list; cc = cc->next) {
     if (cc != uic) {
-      if (uic->stat_win) {
-	gtk_widget_hide(uic->stat_win);
-      }
       if (uic->cwin.top_win) {
 	gtk_widget_hide(uic->cwin.top_win);
       }
@@ -384,11 +409,7 @@ focus_in(GtkIMContext *ic)
   }
   
   uim_helper_client_focus_in(uic->uc);
-  
-  update_status_window(uic);
-  if (uic->stat_win) {
-    gtk_widget_show(uic->stat_win);
-  }
+
 }
 static void
 focus_out(GtkIMContext *ic)
@@ -398,16 +419,11 @@ focus_out(GtkIMContext *ic)
   if(focused_context == uic)
     focused_context = NULL;
 
-  if (uic->win && uic->stat_win) {
-    gtk_widget_hide(uic->stat_win);
-  }
-
   uim_helper_client_focus_out(uic->uc);
 
   if (uic->win) {
     gdk_window_remove_filter(uic->toplevel, toplevel_window_candidate_cb, uic);
   }
-
   if(uic->cwin.top_win){
     gtk_widget_hide(uic->cwin.top_win);
   }
@@ -498,7 +514,6 @@ toplevel_window_candidate_cb (GdkXEvent *xevent, GdkEvent *ev, gpointer data)
   if(uic->cwin.top_win && uic->cwin.is_active ){
     layout_candidate(uic);
   }
-  update_status_window(uic);
   return GDK_FILTER_CONTINUE;
 }
 
@@ -525,7 +540,6 @@ set_client_window(GtkIMContext *ic, GdkWindow *w)
   }
 
   uic->win = w;
-  update_status_window(uic);
 }
 
 
@@ -544,13 +558,9 @@ im_uim_finalize(GObject *obj)
   if (uic->menu) {
     gtk_widget_destroy(uic->menu);
   }
-  if (uic->stat_win) {
-    gtk_widget_destroy(uic->stat_win);
-  }
   if (uic->cwin.top_win) {
     gtk_widget_destroy(uic->cwin.top_win);
   }
-  uic->stat_win = NULL;
   uic->menu = NULL;
 
   uim_release_context(uic->uc);
@@ -602,6 +612,9 @@ update_prop_label_cb(void *ptr, char *str)
   g_string_free(tmp, TRUE);
 }
 
+
+/* This function is called by Gtk+, when user select candidate
+   by clicking candidate, etc. */
 static void
 select_candidate_cb(GtkWidget *clist,
 		    gint row, gint column,
@@ -645,6 +658,8 @@ init_candidate_win(IMUIMContext *uic)
   } else {
     cwin->left = FALSE;
   }
+  g_free(leftp);
+
   cwin->top_win = gtk_window_new(GTK_WINDOW_POPUP);
   cwin->clist = gtk_clist_new(2);
   cwin->is_active = 0;
@@ -681,21 +696,40 @@ trim_index(IMUIMContext *uic)
 {
   struct candidate_window *cwin = &uic->cwin;
   int changed = 0;
-  while (cwin->layout_begin + NR_CANDIDATES <= cwin->candidate_index) {
-    changed = 1;
-    cwin->layout_begin += NR_CANDIDATES;
-  }  
-  while (cwin->layout_begin > cwin->candidate_index) {
-    changed = 1;
-    cwin->layout_begin -= NR_CANDIDATES;
+
+  if(cwin->display_limit != 0){
+    
+    while (cwin->layout_begin + cwin->display_limit <= cwin->candidate_index) {
+      changed = 1;
+      cwin->layout_begin += cwin->display_limit;
+    }  
+    while (cwin->layout_begin > cwin->candidate_index) {
+      changed = 1;
+      cwin->layout_begin -= cwin->display_limit;
+    }
+    if (cwin->layout_begin < 0) {
+      changed = 1;
+      cwin->layout_begin = 0;
+    }
   }
-  if (cwin->layout_begin < 0) {
-    changed = 1;
-    cwin->layout_begin = 0;
+  else {
+    while (cwin->layout_begin + NR_CANDIDATES <= cwin->candidate_index) {
+      changed = 1;
+      cwin->layout_begin += NR_CANDIDATES;
+    }  
+    while (cwin->layout_begin > cwin->candidate_index) {
+      changed = 1;
+      cwin->layout_begin -= NR_CANDIDATES;     
+    }
+    if (cwin->layout_begin < 0) {
+      changed = 1;
+      cwin->layout_begin = 0;
+    }
   }
   return changed;
 }
 
+
 static void
 calc_candidate_window_size(struct candidate_window cwin, int *width, int *height)
 {
@@ -710,7 +744,7 @@ calc_candidate_window_size(struct candidate_window cwin, int *width, int *height
   if(*width < 80)
     *width = 80;
 
-  gtk_window_resize(GTK_WINDOW(cwin.top_win), *width, *height);
+  gtk_window_resize(cwin.top_win, *width, *height);
 }
 
 static void
@@ -720,8 +754,8 @@ layout_candidate(IMUIMContext *uic)
   int  sc_he, cw_he; /*screen height, candidate window height*/
   int  sc_wi, cw_wi;
   char idx[20];
-  char *new_cand = NULL;
-  char *old_cand = NULL;
+  char *old_cand_str = NULL;
+  char *new_cand_str = NULL;
 
   trim_index(uic);
 
@@ -746,32 +780,42 @@ layout_candidate(IMUIMContext *uic)
 
   gtk_window_move(GTK_WINDOW(uic->cwin.top_win), x, y );
   
-  for (i = 0; i < NR_CANDIDATES; i++) {
+  /* XXX: Consider if display_limit == 0 */
+  for (i = 0; i < NR_CANDIDATES && i < uic->cwin.display_limit; i++) {
     if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
-      new_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin + i);
-      gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), i, 1, &old_cand);
-
+      uim_candidate new_cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin + i, i);
       g_return_if_fail(new_cand);
+      
+      new_cand_str = (gchar*)uim_candidate_get_cand_str(new_cand);
+      g_return_if_fail(new_cand_str);
 
-      if(!old_cand || strcmp(new_cand, old_cand) != 0) {
+      gtk_clist_get_text(GTK_CLIST(uic->cwin.clist), i, 1, &old_cand_str);
+
+      /* Checking old cand and new cand is the same or not. If not, call update. 
+	 Caching new result is good idea for speed up and should be implement.*/
+      if(!old_cand_str || strcmp(new_cand_str, old_cand_str) != 0) {
 	update_candidate(uic);
-	free(new_cand);
+	uim_candidate_free(new_cand);
 	break;
       }
-      free(new_cand);
+      uim_candidate_free(new_cand);
     }
   }
   
+  uic->cwin.select_count ++;
 
   if(uic->cwin.candidate_index >= 0) {
-  uic->cwin.select_count ++;
-  gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
-		       uic->cwin.candidate_index - uic->cwin.layout_begin,
-		       0);
+    gtk_clist_select_row(GTK_CLIST(uic->cwin.clist),
+			 uic->cwin.candidate_index - uic->cwin.layout_begin,
+			 0);
+    
+    sprintf(idx, "%d / %d", uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates); 
+  } else {
+    gtk_clist_unselect_all(GTK_CLIST(uic->cwin.clist));
+    sprintf(idx, "- / %d", uic->cwin.nr_candidates); 
   }
-
-  sprintf(idx, "%d / %d",uic->cwin.candidate_index + 1 , uic->cwin.nr_candidates);  
   gtk_label_set_text( GTK_LABEL(uic->cwin.numlabel), idx);
+  
 }
 
 
@@ -779,51 +823,74 @@ static void
 update_candidate(IMUIMContext *uic)
 {
   char *buf[2];
-  char idx[20];
   int i;
-
+  uim_candidate cand = NULL;
   gtk_clist_freeze(GTK_CLIST(uic->cwin.clist));
   gtk_clist_clear(GTK_CLIST(uic->cwin.clist));
   
-  for (i = 0; i < NR_CANDIDATES; i++) {
+  /* XXX: Consider if display_limit == 0 */
+  for (i = 0; i < NR_CANDIDATES && i < uic->cwin.display_limit; i++) {
     if (i < uic->cwin.nr_candidates - uic->cwin.layout_begin) {
-      g_sprintf(idx, "%2d", i + uic->cwin.layout_begin + 1 );
-      buf[0] = idx;
-      buf[1] = uim_get_candidate(uic->uc, i + uic->cwin.layout_begin);
+      cand = uim_get_candidate(uic->uc, uic->cwin.layout_begin + i, i);
+      buf[0] = uim_candidate_get_heading_label(cand);
+      buf[1] = uim_candidate_get_cand_str(cand);
+      gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
+      uim_candidate_free(cand);
     } else {
       buf[0] = "";
       buf[1] = "";
+      gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
     }
-    gtk_clist_append(GTK_CLIST(uic->cwin.clist), buf);
-    
-    if(buf[1] && strcmp(buf[1],"") != 0 )
-      free(buf[1]);
   }
   gtk_clist_thaw( GTK_CLIST(uic->cwin.clist) );
-  
 }
 
 static void
-cand_begin_cb(void *ptr, int nr, int index)
+cand_begin_cb(void *ptr, int nr, int display_limit)
 {
   IMUIMContext *uic = ptr;
   init_candidate_win(uic);
   uic->cwin.is_active = 1;
-  uic->cwin.nr_candidates = nr;
   uic->cwin.candidate_index = -1;
+  uic->cwin.nr_candidates = nr;
+  uic->cwin.display_limit = display_limit;
   uic->cwin.layout_begin = 0;
   layout_candidate(uic);
   gtk_widget_show(uic->cwin.top_win);
 }
 
+
+/* This function called by libuim.
+   Selected by keyboard, etc. */
 static void
-cand_update_cb(void *ptr, int index)
+cand_select_cb(void *ptr, int index)
 {
   IMUIMContext *uic = ptr;
   uic->cwin.candidate_index = index;
   layout_candidate(uic);
 }
 
+static void
+cand_shift_page_cb(void *ptr, int direction)
+{
+  IMUIMContext *uic = ptr;
+
+  if(direction){
+    if(uic->cwin.nr_candidates < uic->cwin.candidate_index + uic->cwin.display_limit )
+      uic->cwin.candidate_index = 0;
+    else
+      uic->cwin.candidate_index += uic->cwin.display_limit;
+  }
+  else {
+    if(uic->cwin.candidate_index - uic->cwin.display_limit < 0)
+      uic->cwin.candidate_index = uic->cwin.nr_candidates - 1;
+    else
+      uic->cwin.candidate_index -= uic->cwin.display_limit;
+  }
+  layout_candidate(uic);
+  uim_set_candidate_index(uic->uc, uic->cwin.candidate_index);
+}
+
 static void
 cand_end_cb(void *ptr)
 {
@@ -831,7 +898,7 @@ cand_end_cb(void *ptr)
   if(uic->cwin.top_win) {
     gtk_widget_hide(uic->cwin.top_win);
   }
-  uic->nr_psegs = 0;
+  /*  uic->nr_psegs = 0; */
   uic->cwin.is_active = 0;
 }
 
@@ -844,7 +911,6 @@ im_module_create (const gchar *context_id)
 
   uic->win = NULL;
   uic->menu = NULL;
-  uic->stat_win = NULL;
   uic->pseg = 0;
   uic->nr_psegs = 0;
 
@@ -860,6 +926,7 @@ im_module_create (const gchar *context_id)
 
   uic->uc =  uim_create_context(uic, "UTF-8",
 				NULL, (char *)context_id,
+				uim_iconv,
 				im_uim_commit_string);
   if (uic->uc == NULL) {
     parent_class->finalize(obj);
@@ -876,10 +943,11 @@ im_module_create (const gchar *context_id)
 			      update_prop_list_cb);
   uim_set_prop_label_update_cb(uic->uc,
 			      update_prop_label_cb);
-  uim_set_candidate_cb(uic->uc,
-		       cand_begin_cb,
-		       cand_update_cb,
-		       cand_end_cb);
+  uim_set_candidate_selector_cb(uic->uc,
+				cand_begin_cb,
+				cand_select_cb,
+				cand_shift_page_cb,
+				cand_end_cb);
   
   uim_prop_list_update(uic->uc);
 
@@ -908,7 +976,7 @@ im_module_list(GtkIMContextInfo ***contexts,
 
   uim_init();
   uc = uim_create_context(NULL, "UTF-8",
-			  NULL, NULL, NULL);
+			  NULL, NULL, uim_iconv, NULL);
   if (uc == NULL)
     nr = 0;
   else
@@ -916,13 +984,12 @@ im_module_list(GtkIMContextInfo ***contexts,
 
   info_list = malloc(sizeof(GtkIMContextInfo*)*nr);
   for (i = 0; i < nr; i++) {
-    char *name = uim_get_im_name(uc, i);
-    char *lang = uim_get_im_language(uc, i);
+    const char *name = uim_get_im_name(uc, i);
+    const char *lang = uim_get_im_language(uc, i);
     char s[100];
 
     info_list[i] = malloc(sizeof(GtkIMContextInfo));
 
-
     sprintf(s, "uim-%s", name);
     info_list[i]->context_id = g_strdup(s);
 
@@ -932,9 +999,6 @@ im_module_list(GtkIMContextInfo ***contexts,
     info_list[i]->domain = "gtk+";
     info_list[i]->domain_dirname = "";
     info_list[i]->default_locales = g_strdup(lang);
-
-    g_free(name);
-    g_free(lang);
   }
   uim_release_context(uc);
   *contexts = info_list;
@@ -958,7 +1022,7 @@ im_uim_parse_helper_str(char *str)
 	g_strfreev(lines);
       }
     } else if(strncmp("focus_in", str, 8) == 0) {
-      focused_context = NULL;
+      /*      focused_context = NULL; */
     }
   }
 }
@@ -1017,8 +1081,7 @@ im_module_init(GTypeModule *type_module)
 				&class_info, 0);
 
   /* XXX:This is not recommended way!! */
-  snooper_id = gtk_key_snooper_install( (GtkKeySnoopFunc*)uim_key_snoop, NULL );
-
+  snooper_id = gtk_key_snooper_install( (GtkKeySnoopFunc)uim_key_snoop, NULL );
 }
 
 void
