@@ -214,7 +214,7 @@ void str_parse(char *str)
     
     if(strcmp("begin", tmp[i]) == 0){
       candwin_begin(tmp);
-    } else if(strcmp("update", tmp[i]) == 0) {
+    } else if(strcmp("select", tmp[i]) == 0) {
       candwin_update(tmp);
     } else if(strcmp("show", tmp[i]) == 0) {
       candwin_show();
