@@ -6,6 +6,9 @@
 ;; configs
 (define anthy-use-candidate-window? #t)
 (define anthy-candidate-op-count 1)
+(define anthy-nr-candidate-max 10)
+(define anthy-show-segment-separator? #f)
+(define anthy-segment-separator "|")
 
 ;; key defs
 (define anthy-on-key
@@ -25,6 +28,11 @@
   (lambda (key key-state)
     (and (= (to-lower-char key) 108)
 	 (shift-key-mask key-state))))
+(define anthy-hankaku-kana-key
+  (lambda (key key-state)
+    (and
+     (= (to-lower-char key) 113)
+     (control-key-mask key-state))))
 (define anthy-kana-toggle-key
   (lambda (key key-state)
     (and
@@ -65,6 +73,12 @@
 (define anthy-prev-candidate-key
   (lambda (key key-state)
     (generic-prev-candidate-key key key-state)))
+(define anthy-next-page-key
+  (lambda (key key-state)
+    (generic-next-page-key key key-state)))
+(define anthy-prev-page-key
+  (lambda (key key-state)
+    (generic-prev-page-key key key-state)))
 (define anthy-cancel-key
   (lambda (key key-state)
     (generic-cancel-key key key-state)))
@@ -196,6 +210,23 @@
   (lambda (ac c)
     (set-car! (nthcdr 11 ac) c)))
 
+(define anthy-context-commit-raw
+  (lambda (ac)
+    (car (nthcdr 12 ac))))
+
+(define anthy-context-set-commit-raw!
+  (lambda (ac c)
+    (set-car! (nthcdr 12 ac) c)))
+
+(define anthy-type-hiragana 0)
+(define anthy-type-katakana 1)
+(define anthy-type-hankana 2)
+
+(define anthy-mode-direct 0)
+(define anthy-mode-hiragana 1)
+(define anthy-mode-katakana 2)
+(define anthy-mode-wide-latin 3)
+(define anthy-mode-hankana 4)
 
 ;; on/off state(compose,input) anthy-context-id string rkc index-list
 ;; cur-seg candidate-window
@@ -203,7 +234,7 @@
  (lambda ()
    (let ((c '())
 	 (rkc (rk-context-new ja-rk-rule #t #f)))
-     (set! c (copy-list '(() () () () () () () () () () () #f #t)))
+     (set! c (copy-list '(() () () () () () () () () () () 0 #t #f)))
      (anthy-context-set-ac-id!
       c
       (if anthy-init-lib-ok?
@@ -214,25 +245,42 @@
      (anthy-context-set-on! c #f)
      c)))
 
+(define anthy-commit-raw
+  (lambda (ac id)
+    (im-commit-raw id)
+    (anthy-context-set-commit-raw! ac #t)))
+
 (define anthy-context-kana-toggle
   (lambda (ac)
-    (anthy-context-set-kana-mode!
-     ac
-     (not (anthy-context-kana-mode ac)))))
+    (let ((kana (anthy-context-kana-mode ac)))
+      (if (= kana anthy-type-hiragana)
+	  (anthy-context-set-kana-mode!
+	   ac anthy-type-katakana))
+      (if (= kana anthy-type-katakana)
+	  (anthy-context-set-kana-mode!
+	   ac anthy-type-hiragana))
+      (if (= kana anthy-type-hankana)
+	  (anthy-context-set-kana-mode!
+	   ac anthy-type-hiragana)))))
 
 (define anthy-make-string
-  (lambda (sl dir kana)
-    (if sl
-	(if dir
-	    (string-append (anthy-make-string (cdr sl) dir kana)
-			   (if kana
-			       (cdar sl)
-			       (caar sl)))
-	    (string-append (if kana
-			       (cdar sl)
-			       (caar sl))
-			   (anthy-make-string (cdr sl) dir kana)))
-	"")))
+  (lambda (sl dir type)
+    (let ((get-str-by-type 
+	   (lambda (l)
+	     (cond
+	      ((= type anthy-type-hiragana)
+	       (caar l))
+	      ((= type anthy-type-katakana)
+	       (car (cdar l)))
+	      ((= type anthy-type-hankana)
+	       (cadr (cdar l)))))))
+      (if sl
+	  (if dir
+	      (string-append (anthy-make-string (cdr sl) dir type)
+			     (get-str-by-type sl))
+	      (string-append (get-str-by-type sl)
+			     (anthy-make-string (cdr sl) dir type)))
+	  ""))))
 
 (define anthy-make-left-string
   (lambda (sl kana)
@@ -242,22 +290,22 @@
   (lambda (sl kana)
     (anthy-make-string sl #f kana)))
 
-(define anthy-get-candidate
-  (lambda (ac nth-seg)
-    (let ((id (anthy-context-ac-id ac))
-	  (idx (nth (anthy-context-index-list ac) nth-seg)))
-      ())))
+;(define anthy-get-candidate
+;  (lambda (ac nth-seg)
+;    (let ((id (anthy-context-ac-id ac))
+;	  (idx (nth (anthy-context-index-list ac) nth-seg)))
+;      ())))
 
 (define anthy-init-handler
   (lambda (id arg)
     (let ((c (find-context id)))
-      (set-context-data! c
-			 (anthy-context-new))
+      (set-context-data! c (anthy-context-new))
       (im-clear-mode-list id)
       (im-pushback-mode-list id "ľ")
       (im-pushback-mode-list id "Ҥ餬")
       (im-pushback-mode-list id "")
       (im-pushback-mode-list id "ѱѿ")
+      (im-pushback-mode-list id "Ⱦѥ")
       (im-update-mode-list id)
       (im-update-mode id 0)
       (anthy-update-prop-list id)
@@ -276,18 +324,21 @@
 	   (ac (context-data c)))
       (anthy-flush ac)
       (cond  ; `case' is not supported by uim
-       ((= mode 0)  ; 'direct'
+       ((= mode anthy-mode-direct)
         (anthy-context-set-on! ac #f)
         (anthy-context-set-wide-latin! ac #f))
-       ((= mode 1)  ; 'hiragana'
+       ((= mode anthy-mode-hiragana)
         (anthy-context-set-on! ac #t)
-        (anthy-context-set-kana-mode! ac #f))
-       ((= mode 2)  ; 'katakana'
+        (anthy-context-set-kana-mode! ac anthy-type-hiragana))
+       ((= mode anthy-mode-katakana)
         (anthy-context-set-on! ac #t)
-        (anthy-context-set-kana-mode! ac #t))
-       ((= mode 3)  ; 'wide-latin'
+        (anthy-context-set-kana-mode! ac anthy-type-katakana))
+       ((= mode anthy-mode-wide-latin)
         (anthy-context-set-on! ac #f)
-        (anthy-context-set-wide-latin! ac #t))) 
+        (anthy-context-set-wide-latin! ac #t))
+       ((= mode anthy-mode-hankana)
+        (anthy-context-set-on! ac #t)
+        (anthy-context-set-kana-mode! ac anthy-type-hankana)))
       (anthy-update-preedit ac id))
     ()))
 
@@ -310,14 +361,16 @@
 
 (define anthy-update-preedit
   (lambda (ac id)
-    (if (anthy-context-on ac)
-	(if (anthy-context-state ac)
-	    (anthy-compose-state-preedit ac id)
-	    (anthy-input-state-preedit ac id))
-	(begin
-	  (im-clear-preedit id)
-	  (im-update-preedit id)))))
-
+    (if (not (anthy-context-commit-raw ac))
+	(if (anthy-context-on ac)
+	    (if (anthy-context-state ac)
+		(anthy-compose-state-preedit ac id)
+		(anthy-input-state-preedit ac id))
+	    (begin
+	      (im-clear-preedit id)
+	      (im-update-preedit id)))
+	(anthy-context-set-commit-raw! ac #f))))
+  
 (define anthy-proc-raw-state
   (lambda (c key key-state)
     (let ((id (context-id c))
@@ -327,7 +380,7 @@
 	    (anthy-begin-input ac)
 	    (anthy-update-mode ac id)
 	    (anthy-update-prop-label ac id))
-	  (im-commit-raw (context-id c))))))
+	  (anthy-commit-raw ac id)))))
 
 (define anthy-make-index-list-rec
   (lambda (n)
@@ -373,8 +426,8 @@
       (anthy-lib-set-string
        ac-id
        (string-append
-		(anthy-make-left-string (anthy-context-left-string ac) #f)
-		(anthy-make-right-string (anthy-context-right-string ac) #f)))
+		(anthy-make-left-string (anthy-context-left-string ac)  anthy-type-hiragana)
+		(anthy-make-right-string (anthy-context-right-string ac) anthy-type-hiragana)))
       (anthy-context-set-index-list!
        ac
        (anthy-make-index-list
@@ -411,26 +464,33 @@
 	   #t)
        (if (anthy-backspace-key key key-state)
 	   (begin
-	     (im-commit-raw id)
+	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
        (if (anthy-delete-key key key-state)
 	   (begin
-	     (im-commit-raw id)
+	     (anthy-commit-raw ac id)
+	     #f)
+	   #t)
+       (if (anthy-hankaku-kana-key key key-state)
+	   (begin 
+	     (anthy-context-set-kana-mode! ac anthy-type-hankana)
+	     (anthy-update-mode ac id)
+	     (anthy-update-prop-label ac id)
+	     #f)
+	   #t)
+       (if (anthy-kana-toggle-key key key-state)
+	   (begin 
+	     (anthy-context-kana-toggle ac)
+	     (anthy-update-mode ac id)
+	     (anthy-update-prop-label ac id)
 	     #f)
 	   #t)
-     (if (anthy-kana-toggle-key key key-state)
-	 (begin 
-	   (anthy-context-kana-toggle ac)
-	   (anthy-update-mode ac id)
-	   (anthy-update-prop-label ac id)
-	   #f)
-	 #t)
        ;; modifiers (except shift) => ignore
        (if (and (modifier-key-mask key-state)
 		(not (shift-key-mask key-state)))
 	   (begin
-	     (im-commit-raw id)
+	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
        ;; direct key => commit
@@ -440,14 +500,13 @@
 	     #f)
 	   #t)
        (set! key-str (charcode->string (to-lower-char key)))
-	   (if (string-find (rk-expect rkc) key-str)
-	   (let ((res))
-	     (set! res (rk-push-key! rkc key-str))
+       (if (string-find (rk-expect rkc) key-str)
+	   (let ((res (rk-push-key! rkc key-str)))
 	     (if res
 		 (anthy-append-string ac res))
 	     #f)
 	   #t)
-       (im-commit-raw id)))))
+       (anthy-commit-raw ac id)))))
 
 (define anthy-has-preedit?
   (lambda (ac)
@@ -544,6 +603,7 @@
 	      id
 	      (string-append
 		(anthy-make-left-string (anthy-context-left-string ac) kana)
+		(rk-pending rkc)
 		(anthy-make-right-string (anthy-context-right-string ac) kana)))
 	     (anthy-flush ac)
 	     #f)
@@ -602,16 +662,20 @@
        (if (and (modifier-key-mask key-state)
 		(not (shift-key-mask key-state)))
 	   (begin
-	     (im-commit-raw id)
+	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
-       (set! key (to-lower-char key))
-       (let ((res)
-	     (key-str (charcode->string key)))
-	 (set! res (rk-push-key! rkc key-str))
-	 (if res
-	     (anthy-append-string ac res)))))))
-  
+;       (set! key (to-lower-char key))
+       (let* ((key-str (charcode->string key))
+	      (pend (rk-pending rkc))
+	      (res (rk-push-key! rkc key-str)))
+	 (if (and res (not (string-equal? (car res) "")))
+	     (anthy-append-string ac res)
+	     (if using-kana-table?
+	     (begin
+	       (anthy-append-string ac (list pend "" ""))
+;	       (anthy-append-string ac (list (rk-pending rkc) "" "")
+))))))))
 
 (define anthy-proc-input-state
   (lambda (ac id key key-state)
@@ -624,6 +688,14 @@
     (let ((ac-id (anthy-context-ac-id ac)))
       (if (< idx nseg)
 	  (begin
+	    (if (and
+		 anthy-show-segment-separator?
+		 (< 0 idx))
+		(im-pushback-preedit
+		 id
+		 (bit-or preedit-separator
+			 preedit-underline)
+		 anthy-segment-separator))
 	    (im-pushback-preedit
 	     id
 	     (if (= idx (anthy-context-cur-seg ac))
@@ -725,7 +797,7 @@
 	    (anthy-context-set-candidate-window! ac #t)
 	    (im-begin-candidate
 	     id
-	     max 0)))
+	     max anthy-nr-candidate-max)))
       (if (anthy-context-candidate-window ac)
 	  (im-update-candidate id n)))))
 
@@ -805,6 +877,16 @@
 	   (anthy-reset-candidate-window ac id)
 	   #f)
 	 #t)
+     (if (anthy-prev-page-key key key-state)
+	 (begin
+	   (im-shift-page-candidate id #f)
+	   #f)
+	 #t)
+     (if (anthy-next-page-key key key-state)
+	 (begin
+	   (im-shift-page-candidate id #t)
+	   #f)
+	 #t)
      (if (and (modifier-key-mask key-state)
 	      (not (shift-key-mask key-state)))
 	 #f
@@ -831,32 +913,42 @@
        (if (and (modifier-key-mask key-state)
 		(not (shift-key-mask key-state)))
 	   (begin
-	     (im-commit-raw id)
+	     (anthy-commit-raw ac id)
 	     #f)
 	   #t)
        (if w
 	   (im-commit id w)
-	   (im-commit-raw id)))
+	   (anthy-commit-raw ac id)))
       ())))
 
 (define anthy-press-key-handler
   (lambda (id key key-state)
     (let* ((c (find-context id))
 	   (ac (context-data c)))
-      (if (anthy-context-on ac)
-	  (if (anthy-context-state ac)
-	      (anthy-proc-compose-state ac id key key-state)
-	      (anthy-proc-input-state ac id key key-state))
-	  (if (anthy-context-wide-latin ac)
-	      (anthy-proc-wide-latin c key key-state)
-	      (anthy-proc-raw-state c key key-state)))
+      (if (and (number? key)
+	       (< key 32))
+	  (im-commit-raw id)
+	  (if (anthy-context-on ac)
+	      (if (anthy-context-state ac)
+		  (anthy-proc-compose-state ac id key key-state)
+		  (anthy-proc-input-state ac id key key-state))
+	      (if (anthy-context-wide-latin ac)
+		  (anthy-proc-wide-latin c key key-state)
+		  (anthy-proc-raw-state c key key-state))))
       ;; preedit
       (anthy-update-preedit ac id))))
 
 
 (define anthy-release-key-handler
   (lambda (id key key-state)
-    ()))
+    (let* ((c (find-context id))
+	   (ac (context-data c)))
+      (if (and
+	   (not (anthy-context-on ac))
+	   (not (anthy-context-wide-latin ac))
+	   (and (number? key) (< key 32)))
+	  ;; don't eat key release event for apps
+	  (anthy-commit-raw ac id)))))
 
 (define anthy-reset-handler
   (lambda (id)
@@ -868,13 +960,13 @@
     (set! anthy-init-lib-ok? #t))
 
 (define anthy-get-candidate-handler
-  (lambda (id idx)
+  (lambda (id idx accel-enum-hint)
     (let* ((c (find-context id))
 	   (ac (context-data c))
-	   (ac-id (anthy-context-ac-id ac)))
-      (anthy-lib-get-nth-candidate
-       ac-id (anthy-context-cur-seg ac)
-       idx))))
+	   (ac-id (anthy-context-ac-id ac))
+	   (cand (anthy-lib-get-nth-candidate
+		  ac-id (anthy-context-cur-seg ac) idx)))
+      (list cand (digit->string (+ idx 1))))))
 
 (define anthy-set-candidate-index-handler
   (lambda (id idx)
@@ -883,6 +975,8 @@
 	   (seg (anthy-context-cur-seg ac))
 	   (ac-id (anthy-context-ac-id ac)))
       (set-car! (nthcdr seg (anthy-context-index-list ac)) idx)
+;      (anthy-reset-candidate-window ac id)
+      (anthy-move-segment ac 1)
       (anthy-update-preedit ac id))))
 
 (define anthy-prop-handler
@@ -891,37 +985,52 @@
 	   (ac (context-data c)))
       (anthy-flush ac)
       (anthy-update-preedit ac id)
-    (if (string-equal? message
+      (cond
+       ((string-equal? message
 		       "prop_anthy_hiragana")
 	(begin
 	  (anthy-context-set-on! ac #t)
-	  (anthy-context-set-kana-mode! ac #t)))
-    (if (string-equal? message
+	  (anthy-context-set-kana-mode! ac anthy-type-hiragana)))
+       ((string-equal? message
 		       "prop_anthy_katakana")
 	(begin
 	  (anthy-context-set-on! ac #t)
-	  (anthy-context-set-kana-mode! ac #f)))
-    (if (string-equal? message
+	  (anthy-context-set-kana-mode! ac anthy-type-katakana)))
+       ((string-equal? message
+		       "prop_anthy_hankana")
+	(begin
+	  (anthy-context-set-on! ac #t)
+	  (anthy-context-set-kana-mode! ac anthy-type-hankana)))
+       ((string-equal? message
 		       "prop_anthy_direct")
 	(begin
-	  (anthy-context-set-on! ac #f)))
+	  (anthy-context-set-on! ac #f)
 	  (anthy-context-set-wide-latin! ac #f)))
-    (if (string-equal? message
+       ((string-equal? message
 		       "prop_anthy_zenkaku")
 	(begin
 	  (anthy-context-set-on! ac #f)
 	  (anthy-context-set-wide-latin! ac #t)))
-    (anthy-update-mode ac id)
-    (anthy-update-prop-label ac id))
+       ((string-equal? message
+		       "prop_anthy_kana")
+	(begin
+	  (load-kana-table))))
+      (anthy-update-mode ac id)
+      (anthy-update-prop-label ac id))))
 
 (define anthy-update-prop-label
   (lambda (ac id)
-    (let ((str ""))
+    (let ((str "")
+	  (kana (anthy-context-kana-mode ac)))
       (set! str
 	    (if (anthy-context-on ac)
-		(if (anthy-context-kana-mode ac)
-		    "\t\n"
-		    "\tҤ餬\n")
+		(cond
+		 ((= kana anthy-type-hiragana)
+		  "\tҤ餬\n")
+		 ((= kana anthy-type-katakana)
+		  "\t\n")
+		  ((= kana anthy-type-hankana)
+		   "\tȾѥ\n"))
 		(if (anthy-context-wide-latin ac)
 		    "\tѱѿ\n"
 		    "a\tľ\n")))
@@ -931,26 +1040,19 @@
 (define anthy-update-mode
   (lambda (ac id)
     (if (anthy-context-on ac)
-	(if (anthy-context-kana-mode ac)
-	    (im-update-mode id 2)
-	    (im-update-mode id 1))
+	(let ((kana (anthy-context-kana-mode ac)))
+	  (cond
+	   ((= kana anthy-type-hiragana)
+	    (im-update-mode id anthy-mode-hiragana))
+	   ((= kana anthy-type-katakana)
+	    (im-update-mode id anthy-mode-katakana))
+	   ((= kana anthy-type-hankana)
+	    (im-update-mode id anthy-mode-hankana))))
 	(if (anthy-context-wide-latin ac)
-	    (im-update-mode id 3)
-	    (im-update-mode id 0)))
+	    (im-update-mode id anthy-mode-wide-latin)
+	    (im-update-mode id anthy-mode-direct)))
     (anthy-update-prop-label ac id)))
 
-(define anthy-prop-list
-  '(
-    ("branch" "Ͼ")
-    ("leaf" "" "Ҥ餬" "Ҥ餬ʤϤǤޤ" "prop_anthy_hiragana")
-    ("leaf" "" "" "" "prop_anthy_katakana")
-    ("leaf" "" "ѱѿ" "ѱѿ⡼" "prop_anthy_zenkaku")
-    ("leaf" "a" "ľ" "ե٥åȤϤǤޤ" "prop_anthy_direct")
-    ("branch" "ϥ⡼")
-    ("leaf" "" "޻" "޻" "prop_anthy_roma")
-    ("leaf" "" "" "" "prop_anthy_kana")
-    ))
-
 (define anthy-update-prop-list
   (lambda (id) 
     (let* ((c (find-context id))
@@ -966,10 +1068,12 @@
 		 str
 		 "leaf\t\tҤ餬\tҤ餬ʤǤ\tprop_anthy_hiragana\n"
 		 "leaf\t\t\tʤϤǤޤ\tprop_anthy_katakana\n"
+		 "leaf\t\tȾѥ\tȾѥʤϤǤޤ\tprop_anthy_hankana\n"
 		 "leaf\tA\tľ\t쥯ȤǤ\tprop_anthy_direct\n"
 		 "leaf\t\tѱѿ\tѱѿ⡼\tprop_anthy_zenkaku\n"
-		 "branch\t\t\nleaf\t\t޻\tdesc\tprop_anthy_roma\n"
-		 "leaf\t\t\t\tdescription\tprop_anthy_kana\n"))
+		 "branch\t\t\n"
+		 "leaf\t\t޻\tdesc\tprop_anthy_roma\n"
+		 "leaf\t\t\t\tprop_anthy_kana\n"))
       (im-update-prop-list id str)
       )))
 
