@@ -63,6 +63,9 @@
 (define canna-init-lib-ok? nil)
 (define canna-use-candidate-window? #t)
 (define canna-candidate-op-count 1)
+(define canna-nr-candidate-max 10)
+(define canna-show-segment-separator? #f)
+(define canna-segment-separator "|")
 ; TODO: support cannaserver on other host
 (define canna-server-name nil)
 ;(define canna-server-name "localhost")
@@ -203,11 +206,17 @@
 (define (canna-context-set-kana-mode! cc c)
   (set-car! (nthcdr 11 cc) c))
 
+(define (canna-context-commit-raw cc c)
+  (car (nthcdr 12 cc)))
+
+(define (canna-context-set-commit-raw! cc c)
+  (set-car! (nthcdr 12 cc) c))
+
 ;;; on/off, state, canna-context-id, string, rkc, index-list, cur-seg, candidate-window, candidate-op-count, wide-latin, kana-mode
 (define (canna-context-new)
   (let ((c '())
 	(rkc (rk-context-new ja-rk-rule #t #f)))
-    (set! c (copy-list '(() () () () () () () () () () () #f #t)))
+    (set! c (copy-list '(() () () () () () () () () () () #f #t #f)))
     (canna-context-set-cc-id! c (if canna-init-lib-ok?
 				    (canna-lib-alloc-context) ()))
     (canna-context-set-rkc! c rkc)
@@ -215,6 +224,10 @@
     (canna-context-set-on! c #f)
     c))
 
+(define (canna-commit-raw cc id)
+  (im-commit-raw id)
+  (canna-context-set-commit-raw! cc #t))
+
 (define (canna-context-kana-toggle cc)
   (canna-context-set-kana-mode! cc (not (canna-context-kana-mode cc))))
 
@@ -507,6 +520,14 @@
   (let ((cc-id (canna-context-cc-id cc)))
     (if (< idx nseg)
 	(begin
+	  (if (and
+	       canna-show-segment-separator?
+	       (< 0 idx))
+	      (im-pushback-preedit
+	       id
+	       (bit-or preedit-separator
+		       preedit-underline)
+	       canna-segment-separator))
 	  (im-pushback-preedit
 	   id
 	   (if (= idx (canna-context-cur-seg cc))
@@ -639,7 +660,7 @@
 	  (canna-context-set-candidate-window! cc #t)
 	  (im-begin-candidate
 	   id
-	   max 0)))
+	   max canna-nr-candidate-max)))
     (if (canna-context-candidate-window cc)
 	(im-update-candidate id n))))
 
@@ -765,19 +786,27 @@
 
 ;;;
 (define (canna-release-key-handler id key key-state)
-  ())
+  (let* ((c (find-context id))
+	 (cc (context-data c)))
+    (if (and
+	 (not (canna-context-on cc))
+	 (not (canna-context-wide-latin cc)))
+	(canna-commit-raw cc id))))
 ;;;
 (define (canna-reset-handler id)
-  ())
+  (let* ((c (find-context id))
+	 (cc (context-data c))
+	 (cc-id (canna-context-cc-id cc)))
+    (canna-lib-reset-conversion cc-id)))
 
 ;;;
-(define (canna-get-candidate-handler id idx)
+(define (canna-get-candidate-handler id idx accel-enum-hint)
   (let* ((c (find-context id))
-	  (cc (context-data c))
-	  (cc-id (canna-context-cc-id cc)))
-    (canna-lib-get-nth-candidate
-     cc-id (canna-context-cur-seg cc)
-     idx)))
+	 (cc (context-data c))
+	 (cc-id (canna-context-cc-id cc))
+	 (cand (canna-lib-get-nth-candidate
+		cc-id (canna-context-cur-seg cc) idx)))
+    (list cand (digit->string (+ idx 1)))))
 
 (define (canna-set-candidate-index-handler id idx)
   (let* ((c (find-context id))
