@@ -14,6 +14,12 @@
      (and
       (= (to-lower-char key) 112)
       (control-key-mask key-state)))))
+(define generic-next-page-key
+  (lambda (key key-state)
+    (= key 'next)))
+(define generic-prev-page-key
+  (lambda (key key-state)
+    (= key 'prior)))
 (define generic-return-key
   (lambda (key key-state)
     (or
