@@ -4,64 +4,69 @@
 
 (define generic-use-candidate-window? #t)
 (define generic-candidate-op-count 1)
+(define generic-nr-candidate-max 10)
+
 (define generic-context-rk-context
   (lambda (pc)
     (car (nthcdr 0 pc))))
-
 (define generic-context-set-rk-context!
   (lambda (pc rkc)
     (set-car! (nthcdr 0 pc) rkc)))
-
 (define generic-context-rk-nth
   (lambda (pc)
     (car (nthcdr 1 pc))))
-
 (define generic-context-set-rk-nth!
   (lambda (pc n)
     (set-car! (nthcdr 1 pc) n)))
-
 (define generic-context-on
   (lambda (pc)
     (car (nthcdr 2 pc))))
-
 (define generic-context-set-on!
   (lambda (pc n)
     (set-car! (nthcdr 2 pc) n)))
-
 (define generic-context-candidate-op-count
   (lambda (pc)
     (car (nthcdr 3 pc))))
-
 (define generic-context-set-candidate-op-count!
   (lambda (pc n)
     (set-car! (nthcdr 3 pc) n)))
-
+(define generic-context-raw-commit
+  (lambda (pc)
+    (car (nthcdr 4 pc))))
+(define generic-context-set-raw-commit!
+  (lambda (pc n)
+    (set-car! (nthcdr 4 pc) n)))
 
 (define generic-update-preedit
   (lambda (id pc)
-    (let* ((rkc (generic-context-rk-context pc))
-	   (cs (rk-current-seq rkc))
-	   (n (generic-context-rk-nth pc)))
-      (im-clear-preedit id)
-      (im-pushback-preedit
-       id preedit-reverse
-       (if cs
-	   (nth n (cadr cs))
-	   (rk-pending rkc)))
-      (im-update-preedit id))))
+    (if (generic-context-raw-commit pc)
+	(generic-context-set-raw-commit! pc #f)
+	(let* ((rkc (generic-context-rk-context pc))
+	       (cs (rk-current-seq rkc))
+	       (n (generic-context-rk-nth pc)))
+	  (im-clear-preedit id)
+	  (im-pushback-preedit
+	   id preedit-reverse
+	   (if cs
+	       (nth n (cadr cs))
+	       (rk-pending rkc)))
+	  (im-update-preedit id)))))
 
 ;; (context nth)
 (define generic-context-new
   (lambda (rule back)
-    (let ((c (copy-list '(() 0 () 0))))
+    (let ((c (copy-list '(() 0 () 0 #f))))
       (generic-context-set-rk-context!
        c (rk-context-new rule #f back))
-      (generic-context-set-rk-nth!
-       c 0)
-      (generic-context-set-on!
-       c #f)
+      (generic-context-set-rk-nth! c 0)
+      (generic-context-set-on! c #f)
       c)))
 
+(define generic-commit-raw
+  (lambda (pc id)
+    (im-commit-raw id)
+    (generic-context-set-raw-commit! pc #t)))
+
 (define generic-proc-on-mode
   (lambda (id pc key state)
     (let* ((rkc (generic-context-rk-context pc))
@@ -79,22 +84,24 @@
 	   #t)
        (if (generic-prev-candidate-key key state)
 	   (begin
-	     (generic-context-set-rk-nth! pc (- n 1))
+	     (set! n (- n 1))
+	     (generic-context-set-rk-nth! pc n)
 	     (if (< n 0)
-		 (generic-context-set-rk-nth! pc (length cs)))
+		 (begin
+		   (generic-context-set-rk-nth! pc (- (length cs) 1))
+		   (set! n (- (length cs) 1))))
 	     (generic-context-set-candidate-op-count!
 	      pc
 	      (+ 1 (generic-context-candidate-op-count pc)))
 	     (if (>= (generic-context-candidate-op-count pc)
 		     generic-candidate-op-count)
-		 (im-update-candidate id (- n 1)))
+		 (im-update-candidate id n))
 	     #f)
 	   #t)
        (if (generic-next-candidate-key key state)
 	   (begin
 	     (generic-context-set-rk-nth! pc (+ 1 n))
-
-	     (if (<= (length cs) n)
+	     (if (<= (length cs) (+ n 1))
 		 (generic-context-set-rk-nth! pc 0))
 	     (generic-context-set-candidate-op-count!
 	      pc
@@ -103,40 +110,41 @@
 		  (= (generic-context-candidate-op-count pc)
 		     generic-candidate-op-count)
 		  generic-use-candidate-window?)
-		 (begin
-		   (im-begin-candidate id (length cs) 0)
-		   (print cs)))
+		 (im-begin-candidate id (length cs) generic-nr-candidate-max))
 	     (if (>= (generic-context-candidate-op-count pc)
 		     generic-candidate-op-count)
 		 (begin
-		   (if (> n (length cs))
-		       (set! n 0))
+		   (if (>= (+ n 1) (length cs))
+		       (set! n -1))
 		   (im-update-candidate id (+ n 1))))
 	     #f)
 	   #t)
        (if (generic-backspace-key key state)
 	   (begin
 	     (if (not (rk-backspace rkc))
-		 (im-commit-raw id))
+		 (generic-commit-raw pc id))
 	     (generic-context-set-rk-nth! pc 0)
+	     (im-end-candidate id)
 	     #f)
 	   #t)
        (if (generic-commit-key key state)
 	   (let ((cs (rk-current-seq rkc)))
-	     (if (> (length (cadr cs)) 1)
+	     (if (> (length (cadr cs)) 0)
 		 (begin
-		   (im-commit id
-			      (nth (generic-context-rk-nth pc) (cadr cs)))
+		   (im-commit id (nth (generic-context-rk-nth pc) (cadr cs)))
 		   (generic-context-set-rk-nth! pc 0)
+		   (generic-context-set-candidate-op-count! pc 0)
 		   (rk-flush rkc)
 		   (im-end-candidate id))
-		 (im-commit-raw id))
+		 (begin
+		   (rk-flush! rkc)
+		   (im-update-preedit id)))
 	     #f)
 	   #t)
        (if (symbol? key)
 	   (begin
 	     (rk-flush rkc)
-	     (im-commit-raw id)
+	     (generic-commit-raw pc id)
 	     (generic-context-set-rk-nth! pc 0)
 	     #f)
 	   #t)
@@ -153,11 +161,16 @@
 		  (im-commit id
 			     (nth (generic-context-rk-nth pc) (cadr cs)))
 		  (generic-context-set-rk-nth! pc 0)
+		  (generic-context-set-candidate-op-count! pc 0)
+		  (im-end-candidate id)
 		  (rk-flush rkc)))))
       (if res
 	  (begin
 	    (im-commit id (nth (generic-context-rk-nth pc) res))
-	    (generic-context-set-rk-nth! pc 0))))))
+	    (generic-context-set-rk-nth! pc 0)
+	    (generic-context-set-candidate-op-count! pc 0)
+	    (im-end-candidate id))
+	  ))))
 
 (define generic-proc-off-mode
   (lambda (id pc key state)
@@ -170,21 +183,28 @@
 	   #f)
 	 #t)
      ;;
-     (im-commit-raw id))))
+     (generic-commit-raw pc id))))
 
 (define generic-key-press-handler
   (lambda (id key state)
     (let* ((c (find-context id))
 	   (pc (context-data c)))
-      (if (generic-context-on pc)
-	  (generic-proc-on-mode id pc key state)
-	  (generic-proc-off-mode id pc key state))
+      (if (and (number? key)
+	       (< key 32))
+	  (im-commit-raw id)
+	  (if (generic-context-on pc)
+	      (generic-proc-on-mode id pc key state)
+	      (generic-proc-off-mode id pc key state)))
       (generic-update-preedit id pc)
       ())))
 
 (define generic-key-release-handler
   (lambda (id key state)
-    ()))
+    (let* ((c (find-context id)))
+      (if (and (not (generic-context-on c))
+	       (and (number? key) (< key 32)))
+	  ;; don't eat key release event for apps
+	  (generic-commit-raw id)))))
 
 (define generic-reset-handler
   (lambda (id)
@@ -202,12 +222,13 @@
       (generic-update-preedit id pc))))
 
 (define generic-get-candidate-handler
-  (lambda (id idx)
+  (lambda (id idx accel-enum-hint)
     (let* ((c (find-context id))
 	   (pc (context-data c))
 	   (rkc (generic-context-rk-context pc))
-	   (cs (cadr (rk-current-seq rkc))))
-      (car (nthcdr idx cs)))))
+	   (cs (cadr (rk-current-seq rkc)))
+	   (cand (car (nthcdr idx cs))))
+      (list cand (digit->string (+ idx 1))))))
 
 (define generic-set-candidate-index-handler
   (lambda (id idx)
