@@ -1,5 +1,7 @@
 ; ܸ/JapaneseEUC-JP
 ;
+(require "util.scm")
+
 ;; im-management
 ;
 (define current-im nil)
@@ -11,13 +13,15 @@
 (define preedit-underline 1)
 (define preedit-reverse 2)
 (define preedit-cursor 4)
+(define preedit-separator 8)
 (define preedit-attr?
   (lambda (attr)
     (or
      (= attr preedit-none)
      (= attr preedit-underline)
      (= attr preedit-reverse)
-     (= attr preedit-cursor))))
+     (= attr preedit-cursor)
+     (= attr preedit-separator))))
 
 ;; config
 (define enable-im-switch #f)
@@ -191,6 +195,10 @@
  default-prop-handler)
 ;;
 ;; Dispatch
+;;
+;; Don't discard unnecessary key events. They are necessary for
+;; proper GUI widget handling. More correction over entire uim
+;; codes is needed.
 (define key-press-handler
   (lambda (id key state)
     (let*
@@ -202,12 +210,17 @@
 	    enable-im-switch
 	    (switch-im-key key state))
 	   (begin
-	     (switch-im id (im-name im)
-			#f)
+	     (switch-im id (im-name im))
+	     #f)
+	   #t)
+       (if (modifier-key key state)
+	   (begin
+	     ;; don't eat modifier press/release edge for apps
+	     (im-commit-raw id)
 	     #t)
-	   (if (not (modifier-key key state))
-	       (handler id key state))
-	   #t)))))
+	   (begin
+	     (handler id key state)
+	     #f))))))
 ;
 (define key-release-handler
   (lambda (id key state)
@@ -215,7 +228,9 @@
 	((c (find-context id))
 	 (im (context-im c))
 	 (handler (im-key-release-handler im)))
-      (if (not (modifier-key key state))
+      (if (modifier-key key state)
+	  ;; don't eat modifier press/release edge for apps
+	  (im-commit-raw id)
 	  (handler id key state)))))
 
 (define reset-handler
@@ -287,6 +302,7 @@
       (im-set-encoding id
 		       (im-encoding im))
       (add-context id im)
+      (update-style uim-color-spec (symbol-value uim-color))
       (handler id arg))))
 
 (define release-context
@@ -300,12 +316,12 @@
       ())))
 
 (define get-candidate
-  (lambda (id idx)
+  (lambda (id idx accel-enum-hint)
     (let*
 	((c (find-context id))
 	 (im (context-im c))
 	 (handler (im-get-candidate-handler im)))
-      (im-return-str (handler id idx)))))
+      (im-return-str-list (handler id idx accel-enum-hint)))))
 (define set-candidate-index
   (lambda (id idx)
     (let*
@@ -314,6 +330,5 @@
 	 (handler (im-set-candidate-index-handler im)))
       (handler id idx))))
 ;;
-(require "util.scm")
 (require "key.scm")
 ;;
