@@ -9,4 +9,6 @@
 (require "viqr.scm")
 (require "hangul.scm")
 (require "ipa.scm")
+(require "romaja.scm")
+(require "pyunihan.scm")
 ;(require "spellcheck.scm")
