@@ -16,20 +16,29 @@
 (require "japanese.scm")
 (require "generic-key.scm")
 
-(define else #t)
-
 ;; configs
 (define prime-use-candidate-window? #t)
 (define prime-candidate-op-count 1) ;;䥦ɥɽޤǤ˲󥹥ڡ򲡤ɬפ뤫
 (define prime-preedit-immediate-commit? #f)
 (define prime-always-show-window? #t)
+(define prime-mask-pending-preedit #f)
+(define prime-nr-candidate-max 10)
+
+;; config function
+(define prime-dont-use-numeral-key-to-select-cand
+  (lambda ()
+    (set! prime-cand-select-key
+	  (lambda (key key-state)
+	    (and (numeral? key)
+		 (control-key-mask key-state))))))
 
 ;; key
 (define prime-latin-key
   (lambda (key key-state)
     (or
      (generic-off-key key key-state)
-     (= (to-lower-char key) 108))))
+     (and (= (to-lower-char key) 108)
+	  (control-key-mask key-state)))))
 (define prime-wide-latin-key
   (lambda (key key-state)
     (and (= (to-lower-char key) 108)
@@ -54,147 +63,123 @@
     (or
      (generic-prev-candidate-key key key-state)
      (= (to-lower-char key) 120))))
-
 (define prime-kana-toggle
   (lambda (key key-state)
-    (= (to-lower-char key) 113)))
+    (and (= (to-lower-char key) 113)
+	 (control-key-mask key-state))))
 (define prime-cancel-key
   (lambda (key key-state)
     (generic-cancel-key key key-state)))
-
 (define prime-backspace-key
   (lambda (key key-state)
     (generic-backspace-key key key-state)))
-
 (define prime-delete-key
   (lambda (key key-state)
     (generic-delete-key key key-state)))
-
 (define prime-go-left-key
   (lambda (key key-state)
     (generic-go-left-key key key-state)))
-
 (define prime-go-right-key
   (lambda (key key-state)
     (generic-go-right-key key key-state)))
+(define prime-cand-select-key
+  (lambda (key key-state)
+    (numeral? key)))
 
 ;; access
+(define prime-context-raw-commit
+ (lambda (ac)
+   (car (nthcdr 16 ac))))
+(define prime-context-set-raw-commit!
+  (lambda (ac mode)
+    (set-car! (nthcdr 16 ac) mode)))
 (define prime-context-immediate-commit ;;Ūprime-preedit-immediate-commit?򥪥դˤ뤫ɤ
  (lambda (ac)
    (car (nthcdr 15 ac))))
-
 (define prime-context-set-immediate-commit!
   (lambda (ac mode)
     (set-car! (nthcdr 15 ac) mode)))
-
 (define prime-context-last-word ;;PRIMEPOBoxѸǤContext
  (lambda (ac)
    (car (nthcdr 14 ac))))
-
 (define prime-context-set-last-word!
   (lambda (ac mode)
     (set-car! (nthcdr 14 ac) mode)))
-
 (define prime-context-mode
  (lambda (ac)
    (car (nthcdr 13 ac))))
-
 (define prime-context-set-mode!
   (lambda (ac mode)
     (set-car! (nthcdr 13 ac) mode)))
-
 (define prime-context-left-string ;;κ¦ˤpreeditʸΥꥹ
  (lambda (ac)
    (car (nthcdr 12 ac))))
-
 (define prime-context-set-left-string!
   (lambda (ac str)
     (set-car! (nthcdr 12 ac) str)))
-
 (define prime-context-right-string
  (lambda (ac)
    (car (nthcdr 11 ac))))
-
 (define prime-context-set-right-string!
   (lambda (ac str)
     (set-car! (nthcdr 11 ac) str)))
-
 (define prime-context-candidates
   (lambda (c)
    (car (nthcdr 10 c))))
-
 (define prime-context-set-candidates!
   (lambda (c cnt)
     (set-car! (nthcdr 10 c) cnt)))
-
 (define prime-context-candidate-window
   (lambda (c)
    (car (nthcdr 9 c))))
-
 (define prime-context-set-candidate-window!
   (lambda (c cnt)
     (set-car! (nthcdr 9 c) cnt)))
-
 (define prime-context-candidate-op-count
   (lambda (c)
    (car (nthcdr 8 c))))
-    
 (define prime-context-set-candidate-op-count!
   (lambda (c cnt)
     (set-car! (nthcdr 8 c) cnt)))
-
 (define prime-context-rk-context
   (lambda (c)
    (car (nthcdr 7 c))))
-
 (define prime-context-set-rk-context!
   (lambda (c rkc)
     (set-car! (nthcdr 7 c) rkc)))
-
 (define prime-context-nth
   (lambda (c)
     (car (nthcdr 6 c))))
-
 (define prime-context-set-nth!
   (lambda (c nth)
     (set-car! (nthcdr 6 c) nth)))
-
 (define prime-context-right-right-strings
   (lambda (c)
     (car (nthcdr 4 c))))
-
 (define prime-context-set-right-right-strings!
   (lambda (c right-right)
     (set-car! (nthcdr 4 c) right-right)))
-
 (define prime-context-left-left-strings
   (lambda (c)
     (car (nthcdr 3 c))))
-
 (define prime-context-set-left-left-strings!
   (lambda (c left-left)
     (set-car! (nthcdr 3 c) left-left)))
-
 (define prime-context-set-learning-word!
   (lambda (c head)
     (set-car! (nthcdr 2 c) head)))
-
 (define prime-context-learning-word
   (lambda (c)
     (car (nthcdr 2 c))))
-
 (define prime-context-kana-mode
   (lambda (sc)
     (car (nthcdr 1 sc))))
-
 (define prime-context-set-kana-mode!
   (lambda (sc mode)
     (set-car! (nthcdr 1 sc) mode)))
-
 (define prime-context-state
   (lambda (c)
     (car (nthcdr 0 c))))
-
 (define prime-context-set-state!
   (lambda (c s)
     (set-car! (nthcdr 0 c) s)))
@@ -217,7 +202,7 @@
   (lambda ()
     (let ((c 
 	   (copy-list 
-	    '(prime-state-latin #t () () () () 0 () () () () () () 0 "" #t))))
+	    '(prime-state-latin #t () () () () 0 () () () () () () 0 "" #t #f))))
       (prime-context-set-rk-context! c
 				   (rk-context-new ja-rk-rule #t #f))
       (prime-flush c)
@@ -232,14 +217,6 @@
 	  (prime-context-set-mode! sc 2))
       (prime-context-set-kana-mode! sc s))))
 
-(define prime-get-string-by-mode
-  (lambda (sc res)
-    (if res
-	(if (prime-context-kana-mode sc)
-	    (car res)
-	    (cdr res))
-	nil)))
-
 (define prime-get-nth-candidate
   (lambda (sc n)
     (if (> n (prime-get-nr-candidates sc))
@@ -272,6 +249,11 @@
        (string-split str "="))
      lst)))
 
+(define prime-commit-raw
+  (lambda (sc id)
+    (im-commit-raw id)
+    (prime-context-set-raw-commit! sc #t)))
+
 (define prime-commit-candidate
   (lambda (sc)
     (let ((nth (prime-context-nth sc))
@@ -328,57 +310,67 @@
 
 (define prime-update-preedit
   (lambda (id sc)
-    (let ((rkc (prime-context-rk-context sc))
-	  (stat (prime-context-state sc))
-	  (learning-word (prime-context-learning-word sc))
-	  (left-left (prime-context-left-left-strings sc))
-	  (right-right (prime-context-right-right-strings sc)))
-      (im-clear-preedit id)
-      (if learning-word
-	  (begin
-	    (im-pushback-preedit
-	     id preedit-reverse
-	     (string-append "[" (string-list-concat learning-word) "|"))
-	    ))
-      (if left-left
-	  (begin
-	    (im-pushback-preedit
-	     id preedit-none
-	      (string-list-concat left-left))))
-      (if (= stat 'prime-state-converting)
-	  (begin
-	    (im-pushback-preedit
-	     id preedit-reverse
-	     (prime-get-current-candidate sc)))
-	  (if (prime-has-preedit? sc)
-	      (let ((hl (string-list-concat (prime-context-left-string sc)))
-		    (hr (string-list-concat (prime-context-right-string sc))))		 
-		(if (string? hl)
-		    (im-pushback-preedit
-		     id preedit-underline
-		     hl))
-		(im-pushback-preedit id preedit-underline
-				     (rk-pending rkc))
-		(im-pushback-preedit id preedit-cursor "")
-		(if (string? hr)
-		    (im-pushback-preedit
-		     id preedit-underline
-		     hr)))
-	      (im-pushback-preedit id preedit-cursor "")))
-
-      (if right-right
-	  (im-pushback-preedit
-	   id preedit-none
-	   (string-list-concat right-right)))
-      (if learning-word
-	  (im-pushback-preedit
-	   id preedit-reverse "]"))
-      (im-update-preedit id))))
+    (if (not (prime-context-raw-commit sc))
+	(let ((rkc (prime-context-rk-context sc))
+	      (stat (prime-context-state sc))
+	      (learning-word (prime-context-learning-word sc))
+	      (left (prime-context-left-string sc))
+	      (right (prime-context-right-string sc))
+	      (left-left (prime-context-left-left-strings sc))
+	      (right-right (prime-context-right-right-strings sc)))
+	  
+	  (if (and
+	       prime-mask-pending-preedit
+	       (usual-char? (string->charcode (car left))))
+	      (set! left (append (list "*") (cdr left))))
+	  
+	  (im-clear-preedit id)
+	  (if learning-word
+	      (begin
+		(im-pushback-preedit
+		 id preedit-reverse
+		 (string-append "[" (string-list-concat learning-word) "|"))))
+	  (if left-left
+	      (begin
+		(im-pushback-preedit
+		 id preedit-none
+		 (string-list-concat left-left))))
+	  (if (= stat 'prime-state-converting)
+	      (begin
+		(im-pushback-preedit
+		 id preedit-reverse
+		 (prime-get-current-candidate sc)))
+	      (if (prime-has-preedit? sc)
+		  (let ((hl (string-list-concat left))
+			(hr (string-list-concat (prime-context-right-string sc))))
+		    (if (string? hl)
+			(im-pushback-preedit
+			 id preedit-underline
+			 hl))
+		    (im-pushback-preedit id preedit-underline
+					 (rk-pending rkc))
+		    (im-pushback-preedit id preedit-cursor "")
+		    (if (string? hr)
+			(im-pushback-preedit
+			 id preedit-underline
+			 hr)))
+		  (im-pushback-preedit id preedit-cursor "")))
+	  
+	  (if right-right
+	      (im-pushback-preedit
+	       id preedit-none
+	       (string-list-concat right-right)))
+	  (if learning-word
+	      (im-pushback-preedit
+	       id preedit-reverse "]"))
+	  (im-update-preedit id))
+	(prime-context-set-raw-commit! sc #f))))
 
 (define prime-update-mode
   (lambda (id sc)
     (let ((mode (prime-context-mode sc)))
-      (im-update-mode id mode))))
+      (im-update-mode id mode)
+      (prime-update-prop-label id sc))))
 
 (define prime-update-candidate-window
   (lambda (sc id)
@@ -403,7 +395,7 @@
 	   preedit
 	   (prime-context-last-word sc))
 	  (im-begin-candidate
-	   id (prime-get-nr-candidates sc) 0)
+	   id (prime-get-nr-candidates sc) prime-nr-candidate-max)
 	  (prime-context-set-candidate-window! sc #t)))
 	  
        ((not (prime-has-preedit? sc))
@@ -459,18 +451,19 @@
 	   (res nil)
 	   (direct (ja-direct (charcode->string key)))
 	   (learning-word (prime-context-learning-word sc))
-	   (cand (string-append (string-list-concat (prime-context-left-left-strings sc))
+	   (registered (string-append (string-list-concat (prime-context-left-left-strings sc))
 				(string-list-concat (prime-context-right-right-strings sc)))))
-	     
       (cond
        ((prime-wide-latin-key key key-state)
 	(begin
 	  (prime-context-set-mode! sc 3)
+	  (prime-context-set-state! sc 'prime-state-wide-latin)
 	  (prime-update-mode id sc)))
        
        ((prime-latin-key key key-state)
 	(begin
 	  (prime-context-set-mode! sc 0)
+	  (prime-context-set-state! sc 'prime-state-latin)
 	  (prime-update-mode id sc)))
        
        ((prime-kana-toggle key key-state)
@@ -483,14 +476,14 @@
 	    (if (prime-context-left-left-strings sc)
 		(prime-context-set-left-left-strings!
 		 sc (cdr (prime-context-left-left-strings sc)))
-		(im-commit-raw id))))
+		(prime-commit-raw sc id))))
 
        ((prime-delete-key key key-state)
 	(if (not (rk-delete rkc))
 	    (if (prime-context-right-right-strings sc)
 		(prime-context-set-right-right-strings!
 		 sc (cdr (prime-context-right-right-strings sc)))
-		(im-commit-raw id))))
+		(prime-commit-raw sc id))))
 
        ((prime-cancel-key key key-state)
 	(if learning-word
@@ -503,14 +496,14 @@
        
        ((prime-commit-key key key-state)
 	(if (and learning-word
-		 (not (string-equal? cand "")))
+		 (not (string-equal? registered "")))
 	    (begin
 	      (prime-lib-send-command
-	       (string-append "learn_word\t" (string-list-concat learning-word) "\t"
-			      cand "\n"))
+	       (string-append "learn_word\t" (string-list-concat learning-word) "\t" registered "\n"))
+	      (im-commit id registered)
 	      (prime-flush sc)
 	      (prime-context-set-learning-word! sc '()))
-	    (im-commit-raw id)))
+	    (prime-commit-raw sc id)))
 	
        ((and (shift-key-mask key-state)
 	     (alphabet-char? key))
@@ -518,7 +511,7 @@
 	  (prime-context-set-immediate-commit! sc #f)
 	  (prime-proc-input-with-preedit c key key-state)))
 
-       ((numeral? key)
+       ((prime-cand-select-key key key-state)
 	(begin
 	  (prime-context-set-immediate-commit! sc #f)
 	  (prime-proc-input-with-preedit c key key-state)))
@@ -531,8 +524,8 @@
 		 sc (cdr (prime-context-left-left-strings sc)))
 		(prime-context-set-right-right-strings!
 		 sc
-		 (append (prime-context-right-right-strings sc) (list c)))))
-	  ))
+		 (append (prime-context-right-right-strings sc) (list c))))
+	      (prime-commit-raw sc id))))
  
        ;; right
        ((prime-go-right-key key key-state)
@@ -543,10 +536,11 @@
 		 sc (reverse (cdr (reverse (prime-context-right-right-strings sc)))))
 		(prime-context-set-left-left-strings!
 		 sc
-		 (append (list c) (prime-context-left-left-strings sc)))))))
+		 (append (list c) (prime-context-left-left-strings sc))))
+	      (prime-commit-raw sc id))))
 
        ((control-key-mask key-state)
-	(im-commit-raw id))
+	(prime-commit-raw sc id))
        
        ;; direct key => commit
        ((and direct
@@ -561,7 +555,7 @@
 	    (begin
 	      (prime-flush sc)
 	      (prime-context-set-last-word! sc "")
-	      (im-commit-raw id))))
+	      (prime-commit-raw sc id))))
 
        (else
 	(begin
@@ -635,25 +629,32 @@
 		 sc
 		 (append (list c) (prime-context-left-string sc)))))))
 		
-       ((and (numeral? key)
-	    (prime-context-immediate-commit sc))
-	(if learning-word
-	    (begin
-	      (prime-context-set-nth! sc (- key 49))
-	      (prime-commit-to-left-left sc (string-to-list (prime-get-nth-candidate sc (prime-context-nth sc)))))
-	    (begin
-	      (prime-context-set-nth! sc (- key 49))
-	      (im-commit id (prime-get-nth-candidate sc (- key 49)))
-	      (prime-commit-candidate sc)
-	      (prime-flush sc)
-	      (prime-update-mode id sc))))
+       ((and (prime-cand-select-key key key-state)
+	     (prime-context-immediate-commit sc))
+	(let* ((nth (- key 49))
+	       (cand (prime-get-nth-candidate sc nth)))
+	  (if cand
+	      (if learning-word
+		  (begin
+		    (prime-context-set-nth! sc nth)
+		    (prime-commit-to-left-left sc (string-to-list cand)))
+		  (begin
+		    (prime-context-set-nth! sc nth)
+		    (im-commit id cand)
+		    (prime-commit-candidate sc)
+		    (prime-flush sc)
+		    (prime-update-mode id sc))))))
 
        ;; modifiers (excepts shift) => ignore
        ((and
 	 (modifier-key-mask key-state)
 	 (not (shift-key-mask key-state)))
-	(im-commit-raw id))
+	(prime-commit-raw sc id))
        
+       ((or (prime-prev-candidate key key-state)
+	    (prime-next-candidate key key-state))
+	(prime-proc-state-converting c key key-state))
+
        (else
 	(begin
 	  (prime-context-set-left-string!
@@ -709,20 +710,21 @@
 	      (prime-commit-candidate sc)
 	      (prime-flush sc)
 	      (prime-update-mode id sc))))
-       
-       ((numeral? key)
-	(if learning-word
-	    (begin
-	      (prime-context-set-nth! sc (- key 49))
-	      (prime-commit-to-left-left
-	       sc
-	       (string-to-list (prime-get-nth-candidate sc (prime-context-nth sc)))))
-	    (begin
-	      (prime-context-set-nth! sc (- key 49))
-	      (im-commit id (prime-get-nth-candidate sc (- key 49)))
-	      (prime-commit-candidate sc)
-	      (prime-flush sc)
-	      (prime-update-mode id sc))))
+
+       ((prime-cand-select-key key key-state)
+	(let* ((nth (- key 49))
+	       (cand (prime-get-nth-candidate sc nth)))
+	  (if cand
+	      (if learning-word
+		  (begin
+		    (prime-context-set-nth! sc nth)
+		    (prime-commit-to-left-left sc (string-to-list cand)))
+		  (begin
+		    (prime-context-set-nth! sc nth)
+		    (im-commit id cand)
+		    (prime-commit-candidate sc)
+		    (prime-flush sc)
+		    (prime-update-mode id sc))))))
 
        (else
 	(begin
@@ -737,7 +739,8 @@
 		(im-commit id (prime-get-current-candidate sc))
 		(prime-commit-candidate sc)
 		(prime-flush sc)
-		(prime-proc-input-no-preedit c key key-state))))))
+		(prime-proc-input-no-preedit c key key-state)))
+	  (prime-update-preedit id sc))))
       )))
 
 
@@ -749,7 +752,7 @@
 	  (begin
 	    (prime-context-set-mode! sc 1)
 	    (prime-update-mode id sc))
-	  (im-commit-raw id)))))
+	  (prime-commit-raw sc id)))))
 
 (define prime-proc-mode-wide-latin
   (lambda (c key key-state)
@@ -763,7 +766,7 @@
 	    (prime-update-mode id sc))
 	  (if w
 	      (im-commit id w)
-	      (im-commit-raw id))))))
+	      (prime-commit-raw sc id))))))
 
 (define prime-push-key
   (lambda (c key key-state)
@@ -797,19 +800,26 @@
       (im-clear-mode-list id)
       (im-pushback-mode-list id "ľ")
       (im-pushback-mode-list id "Ҥ餬")
-      (im-pushback-mode-list id "")
       (im-pushback-mode-list id "ѱѿ")
       (im-update-mode-list id)
-      (im-update-mode id 0))))
+      (im-update-mode id 0)
+
+      (prime-update-prop-list id))))
 
 (define prime-press-key-handler
   (lambda (id key state)
     (let* ((c (find-context id)))
-      (prime-push-key c key state))))
+      (if (and (number? key)
+	       (< key 32))
+	  (im-commit-raw id)
+	  (prime-push-key c key state)))))
 
 (define prime-release-key-handler
   (lambda (id key state)
-    ()))
+    (if (and (number? key)
+	     (< key 32))
+	(im-commit-raw id)
+	())))
 
 (define prime-reset-handler
   (lambda (id)
@@ -829,10 +839,10 @@
       ())))
 
 (define prime-get-candidate-handler
-  (lambda (id idx)
+  (lambda (id idx accel-enum-hint)
     (let* ((c (find-context id))
 	   (sc (context-data c)))
-      (prime-get-nth-candidate sc idx))))
+      (list (prime-get-nth-candidate sc idx) (digit->string (+ idx 1))))))
 
 (define prime-set-candidate-index-handler
   (lambda (id idx)
@@ -841,6 +851,62 @@
       (prime-context-set-nth! sc idx)
       (prime-update-preedit id sc))))
 
+(define prime-prop-handler
+  (lambda (id message)
+    (let* ((c (find-context id))
+	   (sc (context-data c)))
+      (prime-flush sc)
+      (prime-update-preedit sc id)
+      (cond
+       ((string-equal? message
+		       "prop_prime_hiragana")
+	(begin
+	  (prime-context-set-kana-mode! sc #t)))
+       ((string-equal? message
+		       "prop_prime_direct")
+	(begin
+	  (prime-context-set-wide-latin! sc #f)))
+       ((string-equal? message
+		       "prop_prime_zenkaku")
+	(begin
+	  (prime-context-set-wide-latin! sc #t))))
+      (prime-update-mode id sc)
+      (prime-update-prop-label id sc))))
+
+(define prime-update-prop-label
+  (lambda (id sc)
+    (let* ((state (prime-context-state sc))
+	   (str ""))
+      (cond
+       ((= state 'prime-state-latin)
+	(set! str "P\tľ\n"))
+       ((= state 'prime-state-wide-latin)
+	(set! str "\tѱѿ\n"))
+       ((= state 'prime-state-no-preedit)
+	(set! str "\tҤ餬\n")))
+      (im-update-prop-label id str))))
+
+(define prime-update-prop-list
+  (lambda (id) 
+    (let* ((c (find-context id))
+	   (sc (context-data c))
+	   (state (prime-context-state sc))
+	   (str ""))
+      (cond
+       ((= state 'prime-state-latin)
+	(set! str "P\tľ\n"))
+       ((= state 'prime-state-wide-latin)
+	(set! str "\tѱѿ\n"))
+       ((= state 'prime-state-no-preedit)
+	(set! str "\tҤ餬\n")))
+      (set! str 
+	    (string-append "branch\t" str
+			   "leaf\t\tҤ餬\tҤ餬ʥ⡼\tprop_prime_hiragana\n"
+			   "leaf\tP\tľ\tľϥ⡼\tprop_prime_latin\n"
+			   "leaf\t\tѱѿ\tѱѿ⡼\tprop_prime_wide_latin\n"))
+      (im-update-prop-list id str)
+      )))
+
 (register-im
  'prime
  "ja"
@@ -854,4 +920,4 @@
  prime-reset-handler
  prime-get-candidate-handler
  prime-set-candidate-index-handler
- nil)
+ prime-prop-handler)
