@@ -78,7 +78,7 @@
 (define skk-editor-commit
   (lambda (ec str)
     (skk-editor-commit-char-list
-     ec (reverse (skk-lib-split-string str)))))
+     ec (reverse (string-to-list str)))))
 				 
 (define skk-editor-commit-raw
   (lambda (ec key key-state)
@@ -90,6 +90,7 @@
 	    (skk-editor-get-right-string ec))))
       (if raw-str
 	  (skk-editor-commit ec raw-str)
+	  ;; not a string
 	  (and
 	   (if (skk-backspace-key key key-state)
 	       (let ((cur (skk-editor-left-string ec)))
@@ -98,14 +99,24 @@
 		      ec (cdr cur)))
 		 #f)
 	       #t)
-	   (if (= key 'return)
+	   (if (skk-return-key key key-state)
 	       (begin
-		 (skk-lib-learn-word
-		  (skk-make-string (skk-context-head sc) #t)
-		  (skk-context-okuri-head sc)
-		  (skk-make-string (skk-context-okuri sc) #t)
-		  str)
+		 (if (< 0 (string-length str))
+		     (begin
+		       (skk-lib-learn-word
+			(skk-make-string (skk-context-head sc) #t)
+			(skk-context-okuri-head sc)
+			(skk-make-string (skk-context-okuri sc) #t)
+			str)
+		       (skk-save-personal-dictionary)))
 		 (skk-commit-editor-context
 		  sc (skk-editor-id ec) str)
 		 #f)
-	       #t))))))
+	       #t)
+	   (if (skk-cancel-key key key-state)
+	       (begin
+		 (skk-back-to-kanji-state sc (skk-editor-id ec))
+		 (skk-context-set-child-context! sc nil)
+		 #f)
+	       #t)
+	   )))))
