@@ -26,7 +26,10 @@
 (define skk-use-candidate-window? #t)
 (define skk-candidate-op-count 0)
 (define skk-use-recursive-learning? #t)
+(define skk-commit-newline-explicitly? #f)  ;; turn into #t provided safe behavior
 (define skk-style 'skk-style-ddskk-like)
+(define skk-nr-candidate-max 10)
+
 ;; key defs
 (define skk-latin-key
   (lambda (key key-state)
@@ -47,6 +50,11 @@
      (and (= (to-lower-char key) 106)
 	  (control-key-mask key-state))
      (generic-on-key key key-state))))
+(define skk-hankaku-kana-key
+  (lambda (key key-state)
+    (and
+     (= (to-lower-char key) 113)
+     (control-key-mask key-state))))
 (define skk-return-key
   (lambda (key key-state)
     (generic-return-key key key-state)))
@@ -62,7 +70,6 @@
     (or
      (generic-prev-candidate-key key key-state)
      (= (to-lower-char key) 120))))
-
 (define skk-kana-toggle-key
   (lambda (key key-state)
     (and
@@ -149,125 +156,102 @@
     (skk-show-candidates-with-okuri?       . #f)))
 
 ;; access
+
+(define skk-context-commit-raw
+  (lambda (c)
+    (car (nthcdr 14 c))))
+(define skk-context-set-commit-raw!
+  (lambda (c raw)
+    (set-car! (nthcdr 14 c) raw)))
 (define skk-context-latin-conv
   (lambda (c)
     (car (nthcdr 13 c))))
-
 (define skk-context-set-latin-conv!
   (lambda (c lc)
     (set-car! (nthcdr 13 c) lc)))
-
 (define skk-context-editor
   (lambda (c)
     (car (nthcdr 12 c))))
-
 (define skk-context-set-editor!
   (lambda (c cnt)
     (set-car! (nthcdr 12 c) cnt)))
-
 (define skk-context-parent-context
   (lambda (c)
     (car (nthcdr 11 c))))
-
 (define skk-context-set-parent-context!
   (lambda (c cnt)
     (set-car! (nthcdr 11 c) cnt)))
-
 (define skk-context-child-context
   (lambda (c)
     (car (nthcdr 10 c))))
-
 (define skk-context-set-child-context!
   (lambda (c cnt)
     (set-car! (nthcdr 10 c) cnt)))
-
 (define skk-context-candidate-window
   (lambda (c)
     (car (nthcdr 9 c))))
-
 (define skk-context-set-candidate-window!
   (lambda (c cnt)
     (set-car! (nthcdr 9 c) cnt)))
-
 (define skk-context-candidate-op-count
   (lambda (c)
     (car (nthcdr 8 c))))
-
 (define skk-context-set-candidate-op-count!
   (lambda (c cnt)
     (set-car! (nthcdr 8 c) cnt)))
-
 (define skk-context-rk-context
   (lambda (c)
     (car (nthcdr 7 c))))
-
 (define skk-context-set-rk-context!
   (lambda (c rkc)
     (set-car! (nthcdr 7 c) rkc)))
-
 (define skk-context-nth
   (lambda (c)
     (car (nthcdr 6 c))))
-
 (define skk-context-set-nth!
   (lambda (c nth)
     (set-car! (nthcdr 6 c) nth)))
-
 (define skk-context-okuri
   (lambda (c)
     (car (nthcdr 4 c))))
-
 (define skk-context-set-okuri!
   (lambda (c tail)
     (set-car! (nthcdr 4 c) tail)))
-
 (define skk-context-okuri-head
   (lambda (c)
     (car (nthcdr 3 c))))
-
 (define skk-context-set-okuri-head!
   (lambda (c okuri-head)
     (set-car! (nthcdr 3 c) okuri-head)))
-
 (define skk-context-set-head!
   (lambda (c head)
     (set-car! (nthcdr 2 c) head)))
-
 (define skk-context-head
   (lambda (c)
     (car (nthcdr 2 c))))
-
 (define skk-context-kana-mode
   (lambda (sc)
     (car (nthcdr 1 sc))))
-
 (define skk-context-set-kana-mode!
   (lambda (sc mode)
     (set-car! (nthcdr 1 sc) mode)))
-
 (define skk-context-state
   (lambda (c)
     (car (nthcdr 0 c))))
-
 (define skk-context-set-state!
   (lambda (c s)
     (set-car! (nthcdr 0 c) s)))
 ;; state kana head okuri tail candidates nth rk
 
+(define skk-type-hiragana 0)
+(define skk-type-katakana 1)
+(define skk-type-hankana 2)
 
-(define skk-setup-style
-  (lambda (style-spec style)
-    (let* ((elem (car style))
-	   (name (car elem))
-	   (val (if (symbol? (cdr elem))
-		    (symbol-value (cdr elem))
-		    (cdr elem)))
-	   (spec (assq name style-spec))
-	   (valid? (symbol-value (cdr spec))))
-      (if (valid? val)
-	  (set-symbol-value! name val))
-      (if (not (null? (cdr style)))
-	  (skk-setup-style style-spec (cdr style))))))
+(define skk-mode-latin 0)
+(define skk-mode-hiragana 1)
+(define skk-mode-katakana 2)
+(define skk-mode-wide-latin 3)
+(define skk-mode-hankana 4)
 
 (define skk-find-root-context
   (lambda (sc)
@@ -320,7 +304,7 @@
 	  (skk-read-personal-dictionary)))
     (let ((sc 
 	   (copy-list 
-	    '(skk-state-latin #t "" "" "" () () () () () () () () () ()))))
+	    '(skk-state-latin 0 "" "" "" () () () () () () () () () #f))))
       (skk-context-set-head! sc ())
       (skk-context-set-rk-context! sc
 				   (rk-context-new ja-rk-rule #t #f))
@@ -335,24 +319,49 @@
 
 (define skk-make-string
   (lambda (sl kana)
+    (let ((get-str-by-type 
+	   (lambda (l)
+	     (cond
+	      ((= kana skk-type-hiragana)
+	       (caar l))
+	      ((= kana skk-type-katakana)
+	       (car (cdar l)))
+	      ((= kana skk-type-hankana)
+	       (cadr (cdar l)))))))
     (if sl
 	(string-append (skk-make-string (cdr sl) kana)
-		       (if kana
-			   (caar sl)
-			   (cdar sl)))
-	"")))
+		       (get-str-by-type sl))
+	""))))
+
+(define skk-opposite-kana
+  (lambda (kana)
+    (cond
+     ((= kana skk-type-hiragana)
+      skk-type-katakana)
+     ((= kana skk-type-katakana)
+      skk-type-hiragana)
+     ((= kana skk-type-hankana)
+      skk-type-hiragana))))  ; different to ddskk's behavior
 
 (define skk-context-kana-toggle
   (lambda (sc)
-    (let ((s (skk-context-kana-mode sc)))
-      (set! s (not s))
-      (skk-context-set-kana-mode! sc s))))
+    (let* ((kana (skk-context-kana-mode sc))
+	   (opposite-kana (skk-opposite-kana kana)))
+      (skk-context-set-kana-mode! sc opposite-kana))))
 
 (define skk-get-string-mode-part
   (lambda (sc res)
-    (if (skk-context-kana-mode sc)
-	(car res)
-	(cdr res))))
+    (let* ((type (skk-context-kana-mode sc))
+	   (get-str-by-type 
+	    (lambda (l)
+	      (cond
+	       ((= type skk-type-hiragana)
+		(car l))
+	       ((= type skk-type-katakana)
+		(car (cdr l)))
+	       ((= type skk-type-hankana)
+		(cadr (cdr l)))))))
+      (get-str-by-type res))))
 
 (define skk-do-get-string-by-mode
   (lambda (sc str)
@@ -360,8 +369,8 @@
 	(if (string? (car str))
 	    (skk-get-string-mode-part sc str)
 	    (string-append
-	     (skk-do-get-string-by-mode sc(car str))
-	     (skk-do-get-string-by-mode sc(cdr str))))
+	     (skk-do-get-string-by-mode sc (car str))
+	     (skk-do-get-string-by-mode sc (cdr str))))
 	"")))
 
 (define skk-get-string-by-mode
@@ -375,11 +384,9 @@
   (lambda (sc n)
     (skk-lib-get-nth-candidate
      n
-     (skk-make-string (skk-context-head sc)
-		      #t)
+     (skk-make-string (skk-context-head sc) skk-type-hiragana)
      (skk-context-okuri-head sc)
-     (skk-make-string
-      (skk-context-okuri sc) #t))))
+     (skk-make-string (skk-context-okuri sc) skk-type-hiragana))))
 
 (define skk-get-current-candidate
   (lambda (sc)
@@ -394,7 +401,9 @@
 	  (skk-editor-commit-raw
 	   (skk-context-editor psc)
 	   key key-state)
-	  (im-commit-raw id)))))
+	  (begin
+	    (skk-context-set-commit-raw! sc #f)
+	    (im-commit-raw id))))))
 
 ;; commit string
 (define skk-commit
@@ -402,9 +411,29 @@
     (let ((psc (skk-context-parent-context sc)))
       (if psc
 	  (skk-editor-commit
-	   (skk-context-editor psc) str)
+	   (skk-context-editor psc) (skk-lib-remove-annotation str))
 	  (im-commit id (skk-lib-remove-annotation str))))))
 
+(define skk-prepare-commit-string
+  (lambda (sc id)
+    (let* ((cand (skk-get-current-candidate sc))
+	   (okuri (skk-make-string (skk-context-okuri sc)
+				   (skk-context-kana-mode sc)))
+	   (res (string-append cand okuri)))
+      (skk-lib-commit-candidate
+       (skk-make-string (skk-context-head sc) skk-type-hiragana)
+       (skk-context-okuri-head sc)
+       (skk-make-string (skk-context-okuri sc) skk-type-hiragana)
+       (skk-context-nth sc))
+      (if (> (skk-context-nth sc) 0)
+	  (skk-save-personal-dictionary))
+      (skk-reset-candidate-window sc id)
+      (skk-flush sc)
+      (skk-context-set-state! sc 'skk-state-direct)
+      (skk-update-mode id sc)
+      res)))
+
+
 (define skk-append-string
   (lambda (sc str)
     (and
@@ -430,10 +459,9 @@
       ;;
       (set! res
 	    (skk-lib-get-entry
-	     (skk-make-string (skk-context-head sc) #t)
+	     (skk-make-string (skk-context-head sc) skk-type-hiragana)
 	     (skk-context-okuri-head sc)
-	     (skk-make-string
-	      (skk-context-okuri sc) #t)))
+	     (skk-make-string (skk-context-okuri sc) skk-type-hiragana)))
       (if res
 	  (begin
 	    (skk-context-set-nth! sc 0)
@@ -484,7 +512,7 @@
 	    (im-pushback-preedit
 	     id skk-preedit-attr-conv-okuri
 	     (skk-make-string (skk-context-okuri sc)
-			    (skk-context-kana-mode sc)))))
+			      (skk-context-kana-mode sc)))))
 
       (if (or
 	   (= stat 'skk-state-okuri)
@@ -534,22 +562,31 @@
 (define skk-update-mode
   (lambda (id sc)
     (let ((stat (skk-context-state sc))
-	  (mode))
+	  (mode)
+	  (kana (skk-context-kana-mode sc)))
       (set! mode
-	    (if (skk-context-kana-mode sc)
-		1
-		2))
+	    (cond
+	     ((= kana skk-type-hiragana)
+	      skk-mode-hiragana)
+	     ((= kana skk-type-katakana)
+	      skk-mode-katakana)
+	     ((= kana skk-type-hankana)
+	      skk-mode-hankana)))
       (if (= stat 'skk-state-latin)
-	  (set! mode 0))
+	  (set! mode skk-mode-latin))
       (if (= stat 'skk-state-wide-latin)
-	  (set! mode 3))
+	  (set! mode skk-mode-wide-latin))
       (im-update-mode id mode))))
 
 (define skk-update-preedit
   (lambda (id sc)
-    (im-clear-preedit id)
-    (skk-do-update-preedit id (skk-find-root-context sc))
-    (im-update-preedit id)))
+    (if (not (skk-context-commit-raw sc))
+	(begin
+	  (im-clear-preedit id)
+	  (skk-do-update-preedit id (skk-find-root-context sc))
+	  (im-update-preedit id))
+	(skk-context-set-commit-raw! sc #f))))
+
 
 ;; called from skk-editor
 (define skk-commit-editor-context
@@ -596,6 +633,13 @@
 	   (skk-context-set-latin-conv! sc #f)
 	   #f)
 	 #t)
+       (if (skk-hankaku-kana-key key key-state)
+	   (begin
+	     (skk-context-set-kana-mode! sc skk-type-hankana)
+	     (skk-update-mode id sc)
+	     (skk-update-prop-label sc id)
+	     #f)
+	   #t)
      (if (skk-kana-toggle-key key key-state)
 	 (begin 
 	   (skk-context-kana-toggle sc)
@@ -660,7 +704,7 @@
        (if (symbol? key)
 	   (begin
 	     (skk-flush sc)
-	     (skk-commit-raw sc (context-id c) key key-state)
+	     (skk-commit-raw sc id key key-state)
 	     #f)
 	   #t)
        (begin
@@ -717,9 +761,11 @@
 	     (skk-flush sc)
 	     (skk-context-set-state! sc 'skk-state-direct)
 	     (skk-update-mode id sc)
-	     (if (skk-return-key key key-state)
-		 (skk-proc-state-direct c key key-state))
-	     #f)
+ 	     (if (skk-return-key key key-state)
+		 (if skk-commit-newline-explicitly?
+		     (skk-commit sc id "\n")
+		     (skk-proc-state-direct c key key-state)))
+ 	     #f)
 	   #t)
        (if (skk-context-latin-conv sc)
           (begin
@@ -744,9 +790,11 @@
 	   #t)
        (if (skk-kana-toggle-key key key-state)
 	   (begin
-            (skk-commit sc id (skk-make-string
-                               (skk-context-head sc)
-			       (not (skk-context-kana-mode sc))))
+	     (if (skk-context-head sc)
+		 (skk-commit sc id (skk-make-string
+				    (skk-context-head sc)
+				    (skk-opposite-kana
+				     (skk-context-kana-mode sc)))))
             (skk-flush sc)
             (skk-context-set-state! sc 'skk-state-direct)
             (skk-update-mode id sc)
@@ -793,12 +841,10 @@
        (im-begin-candidate
 	id
 	(skk-lib-get-nr-candidates
-	 (skk-make-string (skk-context-head sc)
-			  #t)
+	 (skk-make-string (skk-context-head sc) skk-type-hiragana)
 	 (skk-context-okuri-head sc)
-	 (skk-make-string
-	  (skk-context-okuri sc) #t))
-	0)))))
+	 (skk-make-string (skk-context-okuri sc) skk-type-hiragana))
+	skk-nr-candidate-max)))))
 
 (define skk-change-candidate-index
   (lambda (sc id incr)
@@ -814,18 +860,19 @@
 	      (skk-context-set-nth! sc (- (skk-context-nth sc) 1))
 	      (skk-context-set-nth! sc (- (skk-lib-get-nr-candidates
 					   (skk-make-string
-					    (skk-context-head sc) #t)
+					    (skk-context-head sc)
+					    skk-type-hiragana)
 					   (skk-context-okuri-head sc)
 					   (skk-make-string
-					    (skk-context-okuri sc) #t))
+					    (skk-context-okuri sc)
+					    skk-type-hiragana))
 					  1)))))
     (if (not (skk-get-current-candidate sc))
 	(begin
 	  (skk-context-set-nth! sc 0)
 	  (if skk-use-recursive-learning?
 	      (begin
-		(if (skk-context-candidate-window sc)
-		    (im-end-candidate id))
+		(skk-reset-candidate-window sc id)
 		(skk-setup-child-context sc id)))))
     (if (not (skk-context-child-context sc))
 	(begin
@@ -836,11 +883,17 @@
 	      (im-update-candidate id (skk-context-nth sc)))))
     #f))
 
+(define skk-reset-candidate-window
+  (lambda (sc id)
+    (if (skk-context-candidate-window sc)
+	(begin
+	  (im-end-candidate id sc)
+	  (skk-context-set-candidate-window! sc #f)))
+    (skk-context-set-candidate-op-count! sc 0)))
 
 (define skk-back-to-kanji-state
   (lambda (sc id)
-    (if (skk-context-candidate-window sc)
-	(im-end-candidate id))
+    (skk-reset-candidate-window sc id)
     (skk-context-set-state! sc 'skk-state-kanji)
     (skk-context-set-okuri-head! sc "")
     (if (car (skk-context-okuri sc))
@@ -871,38 +924,21 @@
 	    (skk-commit-key key key-state)
 	    (skk-return-key key key-state))
 	   (begin
-	     (set! res (skk-get-current-candidate sc))
-	     (set!
-	      res
-	      (string-append res
-			     (skk-make-string (skk-context-okuri sc)
-					      (skk-context-kana-mode sc))))
-	     (skk-lib-commit-candidate
-	      (skk-make-string (skk-context-head sc) #t)
-	      (skk-context-okuri-head sc)
-	      (skk-make-string
-	       (skk-context-okuri sc) #t)
-	      (skk-context-nth sc))
-	     (if (> (skk-context-nth sc) 0)
-		 (skk-save-personal-dictionary))
-	     (if (skk-context-candidate-window sc)
-		 (im-end-candidate id))
-	     (skk-flush sc)
-	     (skk-context-set-state! sc 'skk-state-direct)
-	     (skk-update-mode id sc)
+	     (set! res (skk-prepare-commit-string sc id))
 	     (if (skk-return-key key key-state)
 		 (begin
 		   (skk-commit sc id res)
 		   (set! res ())
-		   (skk-proc-state-direct c key key-state)))
+		   (if skk-commit-newline-explicitly?
+		       (skk-commit sc id "\n")
+		       (skk-proc-state-direct c key key-state))))
 	     #f)
 	   #t)
        (begin
 	 (skk-context-set-state! sc 'skk-state-direct)
 	 (skk-update-mode id sc)
 	 (set! res (skk-get-current-candidate sc))
-	 (if (skk-context-candidate-window sc)
-	     (im-end-candidate id))
+	 (skk-reset-candidate-window sc id)
 	 (set!
 	  res
 	  (string-append res 
@@ -1014,7 +1050,7 @@
       ())))
 
 (define skk-push-key
-  (lambda (c key key-state)
+  (lambda (c id key key-state)
     (let* ((sc (skk-current-context c))
 	   (state (skk-context-state sc))
 	   (fun)
@@ -1033,9 +1069,8 @@
 	  (set! fun skk-proc-state-wide-latin))
       (set! res (fun c key key-state))
       (if res
-	  (skk-commit sc (context-id c) res))
-      (skk-update-preedit
-       (context-id c) sc)
+	  (skk-commit sc id res))
+      (skk-update-preedit id sc)
       )))
 
 (define skk-init-handler
@@ -1044,23 +1079,33 @@
       (set-context-data! c
 			 (skk-context-new id))
       (im-clear-mode-list id)
-      (im-pushback-mode-list id "RAW")
+      (im-pushback-mode-list id "ľ")
       (im-pushback-mode-list id "Ҥ餬")
       (im-pushback-mode-list id "")
       (im-pushback-mode-list id "ѱѿ")
+      (im-pushback-mode-list id "Ⱦѥ")
       (im-update-mode-list id)
-      (im-update-mode id 0)
+      (im-update-mode id skk-mode-latin)
       (skk-update-prop-list id)
-      (skk-setup-style skk-style-spec (symbol-value skk-style)))))
+      (update-style skk-style-spec (symbol-value skk-style)))))
 
 (define skk-press-key-handler
   (lambda (id key state)
     (let ((c (find-context id)))
-      (skk-push-key c key state))))
+      (if (and (number? key) (< key 32))
+	  (im-commit-raw id)
+	  (skk-push-key c id key state)))))
 
 (define skk-release-key-handler
   (lambda (id key state)
-    ()))
+    (let* ((c (find-context id))
+	   (sc (skk-current-context c))
+	   (state (skk-context-state sc)))
+      (if (= state 'skk-state-latin)
+	  ;; don't eat key release event for apps
+	  (begin
+	    (skk-context-set-commit-raw! sc #f)
+	    (im-commit-raw id))))))
 
 (define skk-reset-handler
   (lambda (id)
@@ -1071,18 +1116,22 @@
     (let* ((c (find-context id))
 	   (sc (context-data c)))
       (skk-flush sc)
-      (if (= mode 0)
+      (if (= mode skk-mode-latin)
 	  (skk-context-set-state! sc 'skk-state-latin))
-      (if (= mode 1)
+      (if (= mode skk-mode-hiragana)
 	  (begin
 	    (skk-context-set-state! sc 'skk-state-direct)
-	    (skk-context-set-kana-mode! sc #t)))
-      (if (= mode 2)
+	    (skk-context-set-kana-mode! sc skk-type-hiragana)))
+      (if (= mode skk-mode-katakana)
 	  (begin
 	    (skk-context-set-state! sc 'skk-state-direct)
-	    (skk-context-set-kana-mode! sc #f)))
-      (if (= mode 3)
+	    (skk-context-set-kana-mode! sc skk-type-katakana)))
+      (if (= mode skk-mode-wide-latin)
 	  (skk-context-set-state! sc 'skk-state-wide-latin))
+      (if (= mode skk-mode-hankana)
+	  (begin
+	    (skk-context-set-state! sc 'skk-state-direct)
+	    (skk-context-set-kana-mode! sc skk-type-hankana)))
       (skk-update-preedit id sc)
       ())))
 
@@ -1092,23 +1141,24 @@
 	   (sc (context-data c))
 	   (dcsc (skk-find-descendant-context sc))
 	   (cand (skk-get-nth-candidate dcsc idx))
-	   (okuri (skk-context-okuri dcsc)))	  
-      (if (and
-	   okuri
-	   skk-show-candidates-with-okuri?)
-	  (string-append cand
-			 (skk-make-string okuri t))
-	  cand))))
-
+	   (okuri (skk-context-okuri dcsc)))
+      (list
+       (if (and
+	    okuri
+	    skk-show-candidates-with-okuri?)
+	   (string-append cand
+			  (skk-make-string okuri skk-type-hiragana))
+	   cand)
+       (digit->string (+ idx 1))))))
 
 (define skk-set-candidate-index-handler
   (lambda (id idx)
     (let* ((c (find-context id))
-	   (sc (context-data c)))
+	   (sc (skk-current-context c)))
       (skk-context-set-nth! sc idx)
+      (skk-commit sc id (skk-prepare-commit-string sc id))
       (skk-update-preedit  id sc))))
 
-
 (define skk-prop-handler
   (lambda (id message)
     (let* ((c (find-context id))
@@ -1141,15 +1191,19 @@
 (define skk-update-prop-label
   (lambda (sc id)
     (let* ((state (skk-context-state sc))
-	   (kana? (skk-context-kana-mode sc))
+	   (kana (skk-context-kana-mode sc))
 	   (str ""))
-      (if kana?
-	  (set! str "\tҤ餬\n")
-	  (set! str "\t\n"))
-      (if (= state 'skk-state-latin)
-	  (set! str "S\tľ\n"))
-      (if (= state 'skk-state-wide-latin)
-	  (set! str "\tѱѿ\n"))
+      (cond
+       ((= state 'skk-state-latin)
+	(set! str "S\tľ\n"))
+       ((= state 'skk-state-wide-latin)
+	(set! str "\tѱѿ\n"))
+       ((= kana skk-type-hiragana)
+	(set! str "\tҤ餬\n"))
+       ((= kana skk-type-katakana)
+	(set! str "\t\n"))
+       ((= kana skk-type-hankana)
+	(set! str "\tȾѥ\n")))
       (im-update-prop-label id str))))
 
 (define skk-update-prop-list
@@ -1158,7 +1212,7 @@
 	   (sc (context-data c))
 	   (state (skk-context-state sc))
 	   (kana? (skk-context-kana-mode sc))
-	   (str "branch\t"))
+	   (str ""))
       (if kana?
 	  (set! str "\tҤ餬\n")
 	  (set! str "\t\n"))
