@@ -187,9 +187,11 @@
 (define (tutcode-key-press-handler id key key-state)
   (let* ((c (find-context id))
 	 (pc (context-data c)))
-    (if (tutcode-context-on? pc)
-	(tutcode-proc-on-mode id pc key key-state)
-	(tutcode-proc-off-mode id pc key key-state))))
+    (if (and (number? key) (< key 32))
+	(im-commit-raw id)
+	(if (tutcode-context-on? pc)
+	    (tutcode-proc-on-mode id pc key key-state)
+	    (tutcode-proc-off-mode id pc key key-state)))))
 
 ;;; TUT-Code IMνԤ
 ;;; @param id IMID
@@ -198,7 +200,7 @@
   (let* ((c (find-context id)))
     (set-context-data! c (tutcode-context-new))
     (im-clear-mode-list id)
-    (im-pushback-mode-list id "RAW")
+    (im-pushback-mode-list id "ľ")
     (im-pushback-mode-list id "TUT")
     (im-pushback-mode-list id "TUT")
     (im-update-mode-list id)
