@@ -3,11 +3,16 @@
 #include <stdlib.h>
 #include "context.h"
 
-extern LISP sym_t;
-
 #define MAX_CONTEXT 256
-#define LIBANTHY_SO	"libanthy.so"
-#define LIBANTHYDIC_SO	"libanthydic.so"
+
+#ifdef __APPLE__
+  #define LIBANTHY_SO    "libanthy.dylib"
+  #define LIBANTHYDIC_SO "libanthydic.dylib"
+#else
+  #define LIBANTHY_SO    "libanthy.so"
+  #define LIBANTHYDIC_SO "libanthydic.so"
+
+#endif /*__APPLE__*/
 
 struct anthy_context;
 
@@ -99,7 +104,7 @@ init_anthy_lib()
   for (i = 0; i < MAX_CONTEXT; i++) {
     context_slot[i].ac = NULL;
   }
-  return sym_t;
+  return siod_true_value();
 }
 
 static LISP
@@ -109,7 +114,7 @@ create_context()
   for (i = 0; i < MAX_CONTEXT; i++) {
     if (!context_slot[i].ac) {
       context_slot[i].ac = api.create_context();
-      return flocons(i);
+      return intcons(i);
     }
   }
   return NIL;
@@ -153,7 +158,7 @@ get_nr_segments(LISP id_)
   }
   api.get_stat(ac, &acs);
 
-  return flocons(acs.nr_segment);
+  return intcons(acs.nr_segment);
 }
 
 static LISP
@@ -172,7 +177,7 @@ get_nr_candidates(LISP id_, LISP nth_)
   if (nth < cs.nr_segment) {
     struct anthy_segment_stat ss;
     api.get_segment_stat(ac, nth, &ss);
-    return flocons(ss.nr_candidate);
+    return intcons(ss.nr_candidate);
   }
   return NIL;
 }
