@@ -68,7 +68,7 @@
 
 #define BUFSIZE 1024
 
-extern LISP sym_t;
+static LISP true_sym;
 
 struct canna_context {
   char diclist[BUFSIZE];
@@ -198,7 +198,7 @@ init_canna_lib(LISP str_)
     context++;
   }
 
-  return sym_t;
+  return true_sym;
 }
 
 static LISP
@@ -237,7 +237,7 @@ create_context() {
 		buf += buflen;
 	    }
 	}
-	return flocons(i);
+	return intcons(i);
      }
      cc++;
   }
@@ -258,7 +258,7 @@ release_context(LISP id_)
 
   if(api.RkCloseContext(cc->rk_context_id) != -1) {
     cc->rk_context_id = -1;
-    return sym_t;
+    return true_sym;
   } else {
     return NIL;
   }
@@ -359,53 +359,7 @@ begin_conversion(LISP id_, LISP str_)
   if(str != NULL)
      free(str);
 
-  return flocons(cc->segment_num);
-}
-
-static LISP
-next_segment(LISP id_)
-{
-  int id = get_c_long(id_);
-  int new_segment;
-  struct canna_context *cc = get_canna_context(id);
-
-  if(cc == NULL)
-     return NIL;
-
-  if(cc->rk_context_id == -1)
-     return NIL;
-
-  new_segment = cc->current_segment_num + 1;
-  if(new_segment >= cc->segment_num)
-      new_segment = 0;
-
-  cc->current_segment_num = api.RkGoTo(cc->rk_context_id, new_segment);
-  _update_status(id);
-
-  return sym_t;
-}
-
-static LISP
-prev_segment(LISP id_)
-{
-  int id = get_c_long(id_);
-  int new_segment;
-  struct canna_context *cc = get_canna_context(id);
-
-  if(cc == NULL)
-     return NIL;
-
-  if(cc->rk_context_id == -1)
-     return NIL;
-
-  new_segment = cc->current_segment_num - 1;
-  if(new_segment < 0)
-      new_segment += cc->segment_num;
-
-  cc->current_segment_num = api.RkGoTo(cc->rk_context_id, new_segment);
-  _update_status(id);
-
-  return sym_t;
+  return intcons(cc->segment_num);
 }
 
 static LISP
@@ -445,7 +399,7 @@ get_nr_segments(LISP id_)
   if(cc->rk_context_id == -1)
      return NIL;
 
-  return flocons(cc->segment_num);
+  return intcons(cc->segment_num);
 }
 
 static LISP
@@ -457,15 +411,15 @@ get_nr_candidate(LISP id_, LISP nth_)
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
-     return;
+     return NIL;
 
   if(cc->rk_context_id == -1)
-     return;
+     return NIL;
 
   api.RkGoTo(cc->rk_context_id, nth);
 
   if(api.RkGetStat(cc->rk_context_id, &stat) == 0)
-    return flocons(stat.maxcand);
+    return intcons(stat.maxcand);
   else
     return NIL;
 }
@@ -479,10 +433,10 @@ resize_segment(LISP id_, LISP s_, LISP nth_)
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
-     return;
+     return NIL;
 
   if(cc->rk_context_id == -1)
-     return;
+     return NIL;
 
   api.RkGoTo(cc->rk_context_id, s);
   api.RkNfer(cc->rk_context_id);
@@ -494,7 +448,7 @@ resize_segment(LISP id_, LISP s_, LISP nth_)
 
   _update_segment(id, cc->current_segment_num);
 
-  return sym_t;
+  return true_sym;
 }
 
 static LISP
@@ -506,10 +460,10 @@ commit_segment(LISP id_, LISP s_, LISP nth_)
   struct canna_context *cc = get_canna_context(id);
 
   if(cc == NULL)
-     return;
+     return NIL;
 
   if(cc->rk_context_id == -1)
-     return;
+     return NIL;
 
   api.RkEndBun(cc->rk_context_id, 1);
   cc->segment_num = -1;
@@ -526,6 +480,7 @@ reset_conversion(LISP id_)
 void
 uim_init_canna(void)
 {
+  true_sym = siod_true_value();
   init_subr_1("canna-lib-init", init_canna_lib);
 
   init_subr_0("canna-lib-alloc-context", create_context);
