@@ -18,6 +18,17 @@ struct cb_q {
   int flushing;
 };
 
+struct uim_candidate_ {
+  char *str;         /* candidate */
+  char *heading_label;
+  char *annotation;
+  /* uim_pos part_of_speech; */
+  /* int freq; */
+  /* int freshness; */
+  /* int formality; */
+  /* char *src_dict; */
+};
+
 struct uim_context_ {
   void *ptr;
   /* internal id */
@@ -25,7 +36,8 @@ struct uim_context_ {
   /* global id for helper */
   int global_id;
 
-  iconv_t conv;
+  struct uim_code_converter *conv_if;
+  void *conv;
   char *encoding;
   /**/
   int commit_raw_flag;
@@ -55,15 +67,16 @@ struct uim_context_ {
   /* property label */
   void (*prop_label_update_cb)(void *ptr, char *str);
   /* candidate window */
-  void (*candidate_begin_cb)(void *ptr, int nr, int index);
-  void (*candidate_update_cb)(void *ptr, int index);
-  void (*candidate_end_cb)(void *ptr);
+  void (*candidate_selector_begin_cb)(void *ptr, int nr, int index);
+  void (*candidate_selector_select_cb)(void *ptr, int index);
+  void (*candidate_selector_shift_page_cb)(void *ptr, int direciton);
+  void (*candidate_selector_end_cb)(void *ptr);
   /**/
   struct cb_q cb_q;
 };
 
 #define CAND_BEGIN_CB 1
-#define CAND_UPDATE_CB 2
+#define CAND_SELECT_CB 2
 #define CAND_END_CB 3
 #define COMMIT_CB 4
 #define PREEDIT_CLEAR_CB 5
@@ -73,6 +86,7 @@ struct uim_context_ {
 #define MODE_LIST_UPDATE_CB 9
 #define PROP_LABEL_UPDATE_CB 10
 #define PROP_LIST_UPDATE_CB 11
+#define CAND_SHIFT_PAGE_CB 12
 
 struct cb {
   int type;
@@ -84,12 +98,17 @@ struct cb {
 /**/
 uim_context
 uim_find_context(int id);
+void uim_init_key_subrs();
 void uim_init_util_subrs();
 void uim_init_im_subrs();
 
 char *uim_get_c_string(LISP str);
 int uim_key_sym_to_int(LISP sym);
-char *uim_code_conv(char *str, iconv_t ic);
+
+int uim_iconv_is_convertible(const char *tocode, const char *fromcode);
+void *uim_iconv_create(const char *tocode, const char *fromcode);
+char *uim_iconv_code_conv(iconv_t ic, const char *str);
+void uim_iconv_release(void *obj);
 
 void uim_eval_string(uim_context, char *str);
 void uim_schedule_cb(uim_context, int type, char *str, int n1, int n2);
