@@ -10,8 +10,6 @@
 static FILE *primer = NULL, *primew = NULL;
 static int prime_pid = 0;
 
-extern LISP sym_t;
-
 static char *prime_command = "prime";
 
 static LISP
@@ -41,7 +39,7 @@ prime_lib_init(void)
   if(prime_pid == 0) {
     return NIL;
   }
-  return sym_t;
+  return siod_true_value();
 }
 
 void
@@ -54,9 +52,11 @@ uim_init_prime(void)
 void
 uim_quit_prime(void)
 {
+  if(primew) {
+    uim_ipc_send_command(&prime_pid, &primer, &primew, prime_command, "close\n");
+    fclose(primew);
+  }
+
   if(primer)
     fclose(primer);
-
-  if(primew)
-    fclose(primew);
 }
