@@ -120,7 +120,7 @@ struct obj
 #define SUBR5(x) (*((*x).storage_as.subr5.f))
 #define SUBRM(x) (*((*x).storage_as.subrm.f))
 #define SUBRF(x) (*((*x).storage_as.subr.f))
-#define FLONM(x) ((*x).storage_as.flonum.data)
+#define INTNM(x) ((*x).storage_as.flonum.data)
 
 #define NIL ((struct obj *) 0)
 #define EQ(x,y) ((x) == (y))
@@ -135,7 +135,7 @@ struct obj
 
 #define tc_nil    0
 #define tc_cons   1
-#define tc_flonum 2
+#define tc_intnum 2
 #define tc_symbol 3
 #define tc_subr_0 4
 #define tc_subr_1 5
@@ -164,11 +164,11 @@ typedef struct obj *LISP;
 typedef LISP (*SUBR_FUNC) (void);
 
 #define CONSP(x)   TYPEP(x,tc_cons)
-#define FLONUMP(x) TYPEP(x,tc_flonum)
+#define INTNUMP(x) TYPEP(x,tc_intnum)
 #define SYMBOLP(x) TYPEP(x,tc_symbol)
 
 #define NCONSP(x)   NTYPEP(x,tc_cons)
-#define NFLONUMP(x) NTYPEP(x,tc_flonum)
+#define NINTNUMP(x) NTYPEP(x,tc_intnum)
 #define NSYMBOLP(x) NTYPEP(x,tc_symbol)
 
 #define TKBUFFERN 5120
@@ -194,26 +194,23 @@ char *get_c_string (LISP x);
 char *get_c_string_dim (LISP x, long *);
 char *try_get_c_string (LISP x);
 long get_c_long (LISP x);
+long nlength(LISP x);
 
 LISP cons (LISP x, LISP y);
 LISP consp (LISP x);
 LISP car (LISP x);
 LISP cdr (LISP x);
 LISP setcar (LISP cell, LISP value);
-LISP flocons (double x);
+LISP intcons (int x);
 LISP numberp (LISP x);
 LISP eql (LISP x, LISP y);
 LISP symcons (char *pname, LISP vcell);
 LISP symbolp (LISP x);
 LISP symbol_boundp (LISP x, LISP env);
 LISP symbol_value (LISP x, LISP env);
-LISP rintern (char *name);
+LISP rintern (const char *name);
 LISP subrcons (long type, char *name, SUBR_FUNC f);
 LISP closure (LISP env, LISP code);
-void gc_protect (LISP * location);
-void gc_protect_n (LISP * location, long n);
-void gc_protect_sym (LISP * location, char *st);
-void gc_unprotect (LISP * location);
 
 void init_subr (char *name, long type, SUBR_FUNC fcn);
 void init_subr_0 (char *name, LISP (*fcn) (void));
@@ -244,13 +241,10 @@ LISP string_length (LISP string);
 LISP string_trim (LISP);
 LISP string_trim_left (LISP);
 LISP string_trim_right (LISP);
-LISP string_upcase (LISP);
-LISP string_downcase (LISP);
 //LISP copy_list_slib (LISP);
 LISP href (LISP, LISP);
 LISP hset (LISP, LISP, LISP);
 LISP equal (LISP, LISP);
-LISP make_list (LISP x, LISP v);
 void set_fatal_exit_hook (void (*fcn) (void));
 LISP intern (LISP x);
 void init_trace (void);
@@ -272,13 +266,12 @@ LISP cdadr (LISP);
 LISP cddar (LISP);
 LISP cdddr (LISP);
 void chk_string (LISP, char **, long *);
-LISP a_true_value (void);
+LISP siod_true_value (void);
 LISP lapply (LISP fcn, LISP args);
 LISP mallocl (void *lplace, long size);
 void put_st (char *st);
 LISP listn (long n,...);
 char *must_malloc (unsigned long size);
-LISP poparg (LISP *, LISP);
 FILE *get_c_file (LISP p, FILE * deflt);
 char *last_c_errmsg (int);
 LISP llast_c_errmsg (int);
@@ -292,7 +285,6 @@ char *safe_strcpy (char *s1, size_t size1, const char *s2);
 char *safe_strcat (char *s1, size_t size1, const char *s2);
 
 size_t safe_strlen (const char *s, size_t size);
-LISP last (LISP);
 LISP funcall1 (LISP, LISP);
 LISP funcall2 (LISP, LISP, LISP);
 LISP apply1 (LISP, LISP, LISP);
@@ -301,11 +293,6 @@ LISP lungetc (LISP i, LISP p);
 LISP lputc (LISP c, LISP p);
 LISP lputs (LISP str, LISP p);
 
-LISP ccall_catch (LISP tag, LISP (*fcn) (void *), void *);
-LISP lref_default (LISP li, LISP x, LISP fcn);
-
-extern LISP sym_t;
-
 extern long siod_verbose_level;
 extern char *siod_lib;
 
@@ -325,14 +312,11 @@ void full_interrupt_poll (int *counter);
 #endif
 
 long no_interrupt (long n);
-void err_ctrl_c (void);
-double myruntime (void);
 void fput_st (FILE * f, char *st);
 void put_st (char *st);
 void gc_for_newcell (void);
 void gc_ms_stats_start (void);
 void gc_ms_stats_end (void);
-void gc_mark (LISP ptr);
 
 void init_subrs_a (void);
 
