@@ -15,8 +15,6 @@
 
 #include "context.h"
 
-extern LISP sym_t;
-
 /*
  * cand : candidate
  */
@@ -119,13 +117,13 @@ open_dic(const char *fn)
   struct stat st;
   int fd;
   void *addr;
-  if (lstat(fn, &st) == -1) {
-    return NULL;
-  }
   fd = open(fn, O_RDONLY);
   if (fd == -1) {
     return NULL;
   }
+  if(fstat(fd, &st) == -1) {
+    return NULL;
+  }
   addr = mmap(0, st.st_size, PROT_READ, MAP_SHARED, fd, 0);
   close(fd);
   if (addr == MAP_FAILED) {
@@ -523,7 +521,7 @@ skk_get_entry(LISP head_, LISP okuri_head_, LISP okuri_)
   struct skk_cand_array *ca;
   ca = find_cand_array_lisp(head_, okuri_head_, okuri_, 0);
   if (ca) {
-    return sym_t;
+    return siod_true_value();
   }
   return NIL;
 }
@@ -556,7 +554,7 @@ skk_get_nr_candidates(LISP head_, LISP okuri_head_, LISP okuri_)
   if (ca) {
     n = ca->nr_cands;
   }
-  return flocons(n);
+  return intcons(n);
 }
 
 static void
@@ -806,7 +804,7 @@ skk_lib_read_personal_dictionary(LISP fn_)
     }
   }
   fclose(fp);
-  return sym_t;
+  return siod_true_value();
 }
 
 static LISP
