@@ -148,7 +148,7 @@ struct user_type_hooks
 
 /* exported global symbol */
 long siod_verbose_level;
-LISP sym_t;
+static LISP sym_t;
 /*  Added by Spencer Kimball for script-fu shit 6/3/97 */
 static FILE *siod_output;
 char *siod_lib;
@@ -171,7 +171,6 @@ static LISP freelist;
 static jmp_buf errjmp;
 static long errjmp_ok;
 static long nointerrupt;
-static long interrupt_differed;
 static LISP oblistvar;
 static LISP eof_val;
 static LISP sym_errobj;
@@ -302,18 +301,40 @@ no_interrupt (long n)
   long x;
   x = nointerrupt;
   nointerrupt = n;
-  if ((nointerrupt == 0) && (interrupt_differed == 1))
-    {
-      interrupt_differed = 0;
-      err_ctrl_c ();
-    }
+
   return (x);
 }
 
-void
-err_ctrl_c (void)
+long
+get_c_long (LISP x)
 {
-  my_err ("control-c interrupt", NIL);
+  if NINTNUMP
+    (x) my_err ("not a number", x);
+  return ((long) INTNM (x));
+}
+
+long
+nlength (LISP obj)
+{
+  LISP l;
+  long n;
+  switch TYPE
+    (obj)
+    {
+    case tc_string:
+      return (strlen (obj->storage_as.string.data));
+    case tc_nil:
+      return (0);
+    case tc_cons:
+      for (l = obj, n = 0; CONSP (l); l = CDR (l), ++n)
+	INTERRUPT_CHECK ();
+      if NNULLP
+	(l) my_err ("improper list to length", obj);
+      return (n);
+    default:
+      my_err ("wta to length", obj);
+      return (0);
+    }
 }
 
 static LISP
@@ -339,6 +360,23 @@ grepl_puts (char *st, void (*repl_puts) (char *))
     (*repl_puts) (st);
 }
 
+static double
+myruntime (void)
+{
+#if HAVE_SYS_TIMES_H
+  double total;
+  struct tms b;
+  times (&b);
+  total = b.tms_utime;
+  total += b.tms_stime;
+  return (total / 60.0);
+#elif defined (G_OS_WIN32)
+  FILETIME creation, exit, kernel, user;
+  GetProcessTimes (GetCurrentProcess (), &creation, &exit, &kernel, &user);
+  return (kernel.dwLowDateTime * 1e7 + user.dwLowDateTime * 1e7);
+#endif
+}
+
 static long
 repl (struct repl_hooks *h)
 {
@@ -365,23 +403,23 @@ repl (struct repl_hooks *h)
       else
 	repl_return_val = x = (*h->repl_eval) (x);
 
-      sprintf (tkbuffer,
+      /*      sprintf (tkbuffer,
 	       "Evaluation took %g seconds (%g in gc) %ld cons work, %g real.\n",
 	       myruntime () - rt,
 	       gc_time_taken,
 	       gc_cells_allocated,
 	       myrealtime () - ct);
       if (siod_verbose_level >= 3)
-	grepl_puts (tkbuffer, h->repl_puts);
+      grepl_puts (tkbuffer, h->repl_puts);
       if (h->repl_print == NULL)
 	{
 	  if (siod_verbose_level >= 2)
 	    lprint (x, NIL);
 	}
       else
-	(*h->repl_print) (x);
+      (*h->repl_print) (x);*/
     }
-
+     
   return (0);
 }
 
@@ -398,7 +436,6 @@ repl_driver (long want_init, struct repl_hooks *h)
     return (2);
   catch_framep = (struct catch_frame *) NULL;
   errjmp_ok = 1;
-  interrupt_differed = 0;
   nointerrupt = 0;
   if (want_init && init_file && (k == 0))
     vload (init_file, 0, 1);
@@ -608,7 +645,7 @@ lreadsharp (struct gen_readio * f)
     case 'f':
       return (NIL);
     case 't':
-      return (flocons (1));
+      return (intcons (1));
     default:
       return (my_err ("readsharp syntax not handled", NIL));
     }
@@ -776,23 +813,6 @@ repl_c_string (char *str,
     return (2);
 }
 
-double
-myruntime (void)
-{
-#if HAVE_SYS_TIMES_H
-  double total;
-  struct tms b;
-  times (&b);
-  total = b.tms_utime;
-  total += b.tms_stime;
-  return (total / 60.0);
-#elif defined (G_OS_WIN32)
-  FILETIME creation, exit, kernel, user;
-  GetProcessTimes (GetCurrentProcess (), &creation, &exit, &kernel, &user);
-  return (kernel.dwLowDateTime * 1e7 + user.dwLowDateTime * 1e7);
-#endif
-}
-
 void
 set_repl_hooks (void (*puts_f) (char *),
 		LISP (*read_f) (void),
@@ -925,6 +945,21 @@ setcdr (LISP cell, LISP value)
   return (CDR (cell) = value);
 }
 
+static LISP
+last (LISP l)
+{
+  LISP v1, v2;
+  v1 = l;
+  v2 = CONSP (v1) ? CDR (v1) : my_err ("bad arg to last", l);
+  while (CONSP (v2))
+    {
+      INTERRUPT_CHECK ();
+      v1 = v2;
+      v2 = CDR (v2);
+    }
+  return (v1);
+}
+
 static LISP
 nconc (LISP a, LISP b)
 {
@@ -953,7 +988,7 @@ stack_limit (LISP amount, LISP silent)
       return (NIL);
     }
   else
-    return (flocons (stack_size));
+    return (intcons (stack_size));
 }
 
 char *
@@ -1082,7 +1117,7 @@ setcar (LISP cell, LISP value)
 }
 
 LISP
-flocons (double x)
+intcons (int x)
 {
   LISP z;
   long n;
@@ -1091,31 +1126,45 @@ flocons (double x)
       (x >= 0) &&
       (n < inums_dim))
     return (inums[n]);
-  NEWCELL (z, tc_flonum);
-  FLONM (z) = x;
+  NEWCELL (z, tc_intnum);
+  INTNM (z) = x;
   return (z);
 }
 
 LISP
 numberp (LISP x)
 {
-  if FLONUMP
+  if INTNUMP
     (x) return (sym_t);
   else
     return (NIL);
 }
 
+
+static LISP
+ash (LISP value, LISP n)
+{
+  long m, k;
+  m = get_c_long (value);
+  k = get_c_long (n);
+  if (k > 0)
+    m = m << k;
+  else
+    m = m >> (-k);
+  return (intcons (m));
+}
+
 static LISP
 plus (LISP x, LISP y)
 {
   if NULLP
     (y)
-      return (NULLP (x) ? flocons (0) : x);
-  if NFLONUMP
+      return (NULLP (x) ? intcons (0) : x);
+  if NINTNUMP
     (x) my_err ("wta(1st) to plus", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to plus", y);
-  return (flocons (FLONM (x) + FLONM (y)));
+  return (intcons (INTNM (x) + INTNM (y)));
 }
 
 static LISP
@@ -1123,43 +1172,43 @@ ltimes (LISP x, LISP y)
 {
   if NULLP
     (y)
-      return (NULLP (x) ? flocons (1) : x);
-  if NFLONUMP
+      return (NULLP (x) ? intcons (1) : x);
+  if NINTNUMP
     (x) my_err ("wta(1st) to times", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to times", y);
-  return (flocons (FLONM (x) * FLONM (y)));
+  return (intcons (INTNM (x) * INTNM (y)));
 }
 
 static LISP
 difference (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to difference", x);
   if NULLP
     (y)
-      return (flocons (-FLONM (x)));
+      return (intcons (-INTNM (x)));
   else
     {
-      if NFLONUMP
+      if NINTNUMP
 	(y) my_err ("wta(2nd) to difference", y);
-      return (flocons (FLONM (x) - FLONM (y)));
+      return (intcons (INTNM (x) - INTNM (y)));
     }
 }
 
 static LISP
 Quotient (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to quotient", x);
   if NULLP
     (y)
-      return (flocons (1 / FLONM (x)));
+      return (intcons (1 / INTNM (x)));
   else
     {
-      if NFLONUMP
+      if NINTNUMP
 	(y) my_err ("wta(2nd) to quotient", y);
-      return (flocons (FLONM (x) / FLONM (y)));
+      return (intcons (INTNM (x) / INTNM (y)));
     }
 }
 
@@ -1167,11 +1216,11 @@ LISP
 lllabs (LISP x)
 {
   double v;
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta to abs", x);
-  v = FLONM (x);
+  v = INTNM (x);
   if (v < 0)
-    return (flocons (-v));
+    return (intcons (-v));
   else
     return (x);
 }
@@ -1179,11 +1228,11 @@ lllabs (LISP x)
 static LISP
 greaterp (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to greaterp", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to greaterp", y);
-  if (FLONM (x) > FLONM (y))
+  if (INTNM (x) > INTNM (y))
     return (sym_t);
   return (NIL);
 }
@@ -1191,11 +1240,11 @@ greaterp (LISP x, LISP y)
 static LISP
 lessp (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to lessp", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to lessp", y);
-  if (FLONM (x) < FLONM (y))
+  if (INTNM (x) < INTNM (y))
     return (sym_t);
   return (NIL);
 }
@@ -1203,11 +1252,11 @@ lessp (LISP x, LISP y)
 LISP
 greaterEp (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to greaterp", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to greaterp", y);
-  if (FLONM (x) >= FLONM (y))
+  if (INTNM (x) >= INTNM (y))
     return (sym_t);
   return (NIL);
 }
@@ -1215,11 +1264,11 @@ greaterEp (LISP x, LISP y)
 LISP
 lessEp (LISP x, LISP y)
 {
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to lessp", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to lessp", y);
-  if (FLONM (x) <= FLONM (y))
+  if (INTNM (x) <= INTNM (y))
     return (sym_t);
   return (NIL);
 }
@@ -1229,11 +1278,11 @@ lmax (LISP x, LISP y)
 {
   if NULLP
     (y) return (x);
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to max", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to max", y);
-  return ((FLONM (x) > FLONM (y)) ? x : y);
+  return ((INTNM (x) > INTNM (y)) ? x : y);
 }
 
 LISP
@@ -1241,11 +1290,11 @@ lmin (LISP x, LISP y)
 {
   if NULLP
     (y) return (x);
-  if NFLONUMP
+  if NINTNUMP
     (x) my_err ("wta(1st) to min", x);
-  if NFLONUMP
+  if NINTNUMP
     (y) my_err ("wta(2nd) to min", y);
-  return ((FLONM (x) < FLONM (y)) ? x : y);
+  return ((INTNM (x) < INTNM (y)) ? x : y);
 }
 
 static LISP
@@ -1285,8 +1334,8 @@ loop:
       a = cdr (a);
       b = cdr (b);
       goto loop;
-    case tc_flonum:
-      return ((FLONM (a) == FLONM (b)) ? sym_t : NIL);
+    case tc_intnum:
+      return ((INTNM (a) == INTNM (b)) ? sym_t : NIL);
     case tc_symbol:
       return (NIL);
     default:
@@ -1312,11 +1361,11 @@ eql (LISP x, LISP y)
 {
   if EQ
     (x, y) return (sym_t);
-  else if NFLONUMP
+  else if NINTNUMP
     (x) return (NIL);
-  else if NFLONUMP
+  else if NINTNUMP
     (y) return (NIL);
-  else if (FLONM (x) == FLONM (y))
+  else if (INTNM (x) == INTNM (y))
     return (sym_t);
   return (NIL);
 }
@@ -1426,7 +1475,7 @@ must_malloc (unsigned long size)
 }
 
 static LISP
-gen_intern (char *name)
+gen_intern (const char *name)
 {
   LISP l, sym, sl;
   char *cname;
@@ -1436,7 +1485,7 @@ gen_intern (char *name)
     {
       hash = 0;
       n = obarray_dim;
-      cname = name;
+      cname = (char *)name;
       while ((c = *cname++))
 	hash = ((hash * 17) ^ c) % n;
       sl = obarray[hash];
@@ -1462,7 +1511,7 @@ gen_intern (char *name)
 }
 
 LISP
-rintern (char *name)
+rintern (const char *name)
 {
   return (gen_intern (name));
 }
@@ -1493,13 +1542,7 @@ closure (LISP env, LISP code)
   return (z);
 }
 
-void
-gc_protect (LISP * location)
-{
-  gc_protect_n (location, 1);
-}
-
-void
+static void
 gc_protect_n (LISP * location, long n)
 {
   struct gc_protected *reg;
@@ -1510,14 +1553,20 @@ gc_protect_n (LISP * location, long n)
   protected_registers = reg;
 }
 
-void
+static void
+gc_protect (LISP * location)
+{
+  gc_protect_n (location, 1);
+}
+
+static void
 gc_protect_sym (LISP * location, char *st)
 {
   *location = rintern (st);
   gc_protect (location);
 }
 
-void
+static void
 gc_unprotect (LISP * location)
 {
   struct gc_protected *reg;
@@ -1718,8 +1767,8 @@ init_storage_1 (void)
       inums = (LISP *) must_malloc (sizeof (LISP) * inums_dim);
       for (j = 0; j < inums_dim; ++j)
 	{
-	  NEWCELL (ptr, tc_flonum);
-	  FLONM (ptr) = j;
+	  NEWCELL (ptr, tc_intnum);
+	  INTNM (ptr) = j;
 	  inums[j] = ptr;
 	}
       gc_protect_n (inums, inums_dim);
@@ -1896,6 +1945,50 @@ looks_pointerp (LISP p)
   return (0);
 }
 
+
+static void
+gc_mark (LISP ptr)
+{
+  struct user_type_hooks *p;
+gc_mark_loop:
+  if NULLP
+    (ptr) return;
+  if ((*ptr).gc_mark)
+    return;
+  (*ptr).gc_mark = 1;
+  switch ((*ptr).type)
+    {
+    case tc_intnum:
+      break;
+    case tc_cons:
+      gc_mark (CAR (ptr));
+      ptr = CDR (ptr);
+      goto gc_mark_loop;
+    case tc_symbol:
+      ptr = VCELL (ptr);
+      goto gc_mark_loop;
+    case tc_closure:
+      gc_mark ((*ptr).storage_as.closure.code);
+      ptr = (*ptr).storage_as.closure.env;
+      goto gc_mark_loop;
+    case tc_subr_0:
+    case tc_subr_1:
+    case tc_subr_2:
+    case tc_subr_2n:
+    case tc_subr_3:
+    case tc_subr_4:
+    case tc_subr_5:
+    case tc_lsubr:
+    case tc_fsubr:
+    case tc_msubr:
+      break;
+    default:
+      p = get_user_type_hooks (TYPE (ptr));
+      if (p->gc_mark)
+	ptr = (*p->gc_mark) (ptr);
+    }
+}
+
 static void
 mark_locations_array (LISP * x, long n)
 {
@@ -1934,7 +2027,7 @@ free_a_cell(LISP ptr)
     case tc_free_cell:
     case tc_cons:
     case tc_closure:
-    case tc_flonum:
+    case tc_intnum:
     case tc_subr_0:
     case tc_subr_1:
     case tc_subr_2:
@@ -2075,49 +2168,6 @@ gc_ms_stats_end (void)
 	     gc_cells_collected);
 }
 
-void
-gc_mark (LISP ptr)
-{
-  struct user_type_hooks *p;
-gc_mark_loop:
-  if NULLP
-    (ptr) return;
-  if ((*ptr).gc_mark)
-    return;
-  (*ptr).gc_mark = 1;
-  switch ((*ptr).type)
-    {
-    case tc_flonum:
-      break;
-    case tc_cons:
-      gc_mark (CAR (ptr));
-      ptr = CDR (ptr);
-      goto gc_mark_loop;
-    case tc_symbol:
-      ptr = VCELL (ptr);
-      goto gc_mark_loop;
-    case tc_closure:
-      gc_mark ((*ptr).storage_as.closure.code);
-      ptr = (*ptr).storage_as.closure.env;
-      goto gc_mark_loop;
-    case tc_subr_0:
-    case tc_subr_1:
-    case tc_subr_2:
-    case tc_subr_2n:
-    case tc_subr_3:
-    case tc_subr_4:
-    case tc_subr_5:
-    case tc_lsubr:
-    case tc_fsubr:
-    case tc_msubr:
-      break;
-    default:
-      p = get_user_type_hooks (TYPE (ptr));
-      if (p->gc_mark)
-	ptr = (*p->gc_mark) (ptr);
-    }
-}
-
 static LISP
 user_gc (LISP args)
 {
@@ -2193,13 +2243,13 @@ gc_info (LISP arg)
     case 0:
       return NIL;
     case 1:
-      return (flocons (nactive_heaps ()));
+      return (intcons (nactive_heaps ()));
     case 2:
-      return (flocons (nheaps));
+      return (intcons (nheaps));
     case 3:
-      return (flocons (heap_size));
+      return (intcons (heap_size));
     case 4:
-      return (flocons (freelist_length ()));
+      return (intcons (freelist_length ()));
     default:
       return (NIL);
     }
@@ -2865,12 +2915,12 @@ lprin1g (LISP exp, struct gen_printio * f)
 	}
       gput_st (f, ")");
       break;
-    case tc_flonum:
-      n = (long) FLONM (exp);
-      if (((double) n) == FLONM (exp))
+    case tc_intnum:
+      n = (long) INTNM (exp);
+      if (((double) n) == INTNM (exp))
 	sprintf (tkbuffer, "%ld", n);
       else
-	sprintf (tkbuffer, "%g", FLONM (exp));
+	sprintf (tkbuffer, "%g", INTNM (exp));
       gput_st (f, tkbuffer);
       break;
     case tc_symbol:
@@ -2946,7 +2996,7 @@ string_length (LISP string)
 {
   if NTYPEP
     (string, tc_string) err_wta_str (string);
-  return (flocons (strlen (string->storage_as.string.data)));
+  return (intcons (strlen (string->storage_as.string.data)));
 }
 
 
@@ -2955,7 +3005,7 @@ string_dim (LISP string)
 {
   if NTYPEP
     (string, tc_string) err_wta_str (string);
-  return (flocons ((double) string->storage_as.string.dim));
+  return (intcons ((double) string->storage_as.string.dim));
 }
 
 static LISP
@@ -3021,10 +3071,9 @@ lread (LISP f)
 static int
 f_getc (FILE * f)
 {
-  long iflag, dflag;
+  long iflag;
   int c;
   iflag = no_interrupt (1);
-  dflag = interrupt_differed;
   c = getc (f);
   no_interrupt (iflag);
   return (c);
@@ -3104,7 +3153,7 @@ lreadtk (char *buffer, long j)
     }
   if (*p)
     goto a_symbol;
-  return (flocons (atof (buffer)));
+  return (intcons (atof (buffer)));
 a_symbol:
   return (rintern (buffer));
 }
@@ -3249,9 +3298,10 @@ vload (char *fname, long cflag, long rflag)
     {
       int iflag;
       iflag = no_interrupt (1);
-      if ((f = fopen (fname, "r")))
+      /*      if ((f = fopen (fname, "r")))
 	fclose (f);
-      else if ((fname[0] != '/'))
+	else*/
+      if ((fname[0] != '/'))
 	{
 	  fnbuf = alloca(strlen(siod_lib) + strlen(fname) + 2);
 	  strcpy (fnbuf, siod_lib);
@@ -3381,6 +3431,12 @@ nullp (LISP x)
     return (NIL);
 }
 
+static LISP
+stringp (LISP x)
+{
+  return (TYPEP (x, tc_string) ? sym_t : NIL);
+}
+
 FILE *
 get_c_file (LISP p, FILE * deflt)
 {
@@ -3398,7 +3454,7 @@ lgetc (LISP p)
 {
   int i;
   i = f_getc (get_c_file (p, stdin));
-  return ((i == EOF) ? NIL : flocons ((double) i));
+  return ((i == EOF) ? NIL : intcons ((double) i));
 }
 
 LISP
@@ -3421,9 +3477,9 @@ lputc (LISP c, LISP p)
   int i;
   FILE *f;
   f = get_c_file (p, siod_output);
-  if FLONUMP
+  if INTNUMP
     (c)
-      i = (int) FLONM (c);
+      i = (int) INTNM (c);
   else
     i = *get_c_string (c);
   flag = no_interrupt (1);
@@ -3444,7 +3500,7 @@ parse_number (LISP x)
 {
   char *c;
   c = get_c_string (x);
-  return (flocons (atof (c)));
+  return (intcons (atof (c)));
 }
 
 static LISP
@@ -3489,7 +3545,7 @@ siod_verbose (LISP arg)
   if NNULLP
     (arg)
       siod_verbose_level = get_c_long (car (arg));
-  return (flocons (siod_verbose_level));
+  return (intcons (siod_verbose_level));
 }
 
 int
@@ -3501,14 +3557,14 @@ siod_verbose_check (int level)
 LISP
 lruntime (void)
 {
-  return (cons (flocons (myruntime ()),
-		cons (flocons (gc_time_taken), NIL)));
+  return (cons (intcons (myruntime ()),
+		cons (intcons (gc_time_taken), NIL)));
 }
 
 LISP
 lrealtime (void)
 {
-  return (flocons (myrealtime ()));
+  return (intcons (myrealtime ()));
 }
 
 LISP
@@ -3556,9 +3612,9 @@ lrand (LISP m)
   res = rand ();
   if NULLP
     (m)
-      return (flocons (res));
+      return (intcons (res));
   else
-    return (flocons (res % get_c_long (m)));
+    return (intcons (res % get_c_long (m)));
 }
 
 LISP
@@ -3569,23 +3625,11 @@ lsrand (LISP s)
 }
 
 LISP
-a_true_value (void)
+siod_true_value (void)
 {
   return (sym_t);
 }
 
-LISP
-poparg (LISP * ptr, LISP defaultv)
-{
-  LISP value;
-  if NULLP
-    (*ptr)
-      return (defaultv);
-  value = car (*ptr);
-  *ptr = cdr (*ptr);
-  return (value);
-}
-
 char *
 last_c_errmsg (int num)
 {
@@ -3607,7 +3651,7 @@ llast_c_errmsg (int num)
   int xerrno = (num < 0) ? errno : num;
   const char *errmsg = strerror (xerrno);
   if (!errmsg)
-    return (flocons (xerrno));
+    return (intcons (xerrno));
   return (rintern ((char *) errmsg));
 }
 
@@ -3666,25 +3710,25 @@ parser_read (LISP ignore)
 static LISP
 bitand (LISP a, LISP b)
 {
-  return (flocons (get_c_long (a) & get_c_long (b)));
+  return (intcons (get_c_long (a) & get_c_long (b)));
 }
 
 static LISP
 bitor (LISP a, LISP b)
 {
-  return (flocons (get_c_long (a) | get_c_long (b)));
+  return (intcons (get_c_long (a) | get_c_long (b)));
 }
 
 static LISP
 bitxor (LISP a, LISP b)
 {
-  return (flocons (get_c_long (a) ^ get_c_long (b)));
+  return (intcons (get_c_long (a) ^ get_c_long (b)));
 }
 
 static LISP
 bitnot (LISP a)
 {
-  return (flocons (~get_c_long (a)));
+  return (intcons (~get_c_long (a)));
 }
 
 
@@ -3715,7 +3759,7 @@ mapcar2 (LISP fcn, LISP in1, LISP in2)
 static LISP
 llength (LISP obj)
 {
-  return (flocons (nlength (obj)));
+  return (intcons (nlength (obj)));
 }
 
 static LISP
@@ -3757,13 +3801,13 @@ llist (LISP l)
 LISP
 lstrspn (LISP str1, LISP str2)
 {
-  return (flocons (strspn (get_c_string (str1), get_c_string (str2))));
+  return (intcons (strspn (get_c_string (str1), get_c_string (str2))));
 }
 
 LISP
 lstrcspn (LISP str1, LISP str2)
 {
-  return (flocons (strcspn (get_c_string (str1), get_c_string (str2))));
+  return (intcons (strcspn (get_c_string (str1), get_c_string (str2))));
 }
 
 
@@ -3801,6 +3845,76 @@ butlast (LISP l)
   return (my_err ("not a list", l));
 }
 
+LISP
+leval_cond (LISP * pform, LISP * penv)
+{
+  LISP args, env, clause, value, next;
+  args = cdr (*pform);
+  env = *penv;
+  if NULLP
+    (args)
+    {
+      *pform = NIL;
+      return (NIL);
+    }
+  next = cdr (args);
+  while NNULLP
+    (next)
+    {
+      clause = car (args);
+      value = leval (car (clause), env);
+      if NNULLP
+	(value)
+	{
+	  clause = cdr (clause);
+	  if NULLP
+	    (clause)
+	    {
+	      *pform = value;
+	      return (NIL);
+	    }
+	  else
+	    {
+	      next = cdr (clause);
+	      while (NNULLP (next))
+		{
+		  leval (car (clause), env);
+		  clause = next;
+		  next = cdr (next);
+		}
+	      *pform = car (clause);
+	      return (sym_t);
+	    }
+	}
+      args = next;
+      next = cdr (next);
+    }
+  clause = car (args);
+  next = cdr (clause);
+  if NULLP
+    (next)
+    {
+      *pform = car (clause);
+      return (sym_t);
+    }
+  value = leval (car (clause), env);
+  if NULLP
+    (value)
+    {
+      *pform = NIL;
+      return (NIL);
+    }
+  clause = next;
+  next = cdr (next);
+  while (NNULLP (next))
+    {
+      leval (car (clause), env);
+      clause = next;
+      next = cdr (next);
+    }
+  *pform = car (clause);
+  return (sym_t);
+}
 
 LISP
 funcall1 (LISP fcn, LISP a1)
@@ -3855,6 +3969,7 @@ init_subrs (void)
   init_subr_1 ("cdr", cdr);
   init_subr_2 ("set-car!", setcar);
   init_subr_2 ("set-cdr!", setcdr);
+  init_subr_1 ("last", last);
   init_subr_2n ("+", plus);
   init_subr_2n ("-", difference);
   init_subr_2n ("*", ltimes);
@@ -3862,6 +3977,7 @@ init_subrs (void)
   init_subr_2n ("min", lmin);
   init_subr_2n ("max", lmax);
   init_subr_1 ("abs", lllabs);
+  init_subr_2 ("ash", ash);
   init_subr_2 (">", greaterp);
   init_subr_2 ("<", lessp);
   init_subr_2 (">=", greaterEp);
@@ -3871,6 +3987,7 @@ init_subrs (void)
   init_subr_2 ("eqv?", eql);
   init_subr_2 ("=", eql);
   init_subr_2 ("assq", assq);
+  init_msubr ("cond", leval_cond);
   init_subr_2 ("delq", delq);
   init_subr_1 ("read", lread);
   init_subr_1 ("parser_read", parser_read);
@@ -3960,6 +4077,7 @@ init_subrs (void)
   init_lsubr ("list", llist);
   init_subr_2 ("strspn", lstrspn);
   init_subr_2 ("strcspn", lstrcspn);
+  init_subr_1 ("string?", stringp);
   init_subr_3 ("ass", ass);
   init_subr_2 ("nth", nth);
   init_subr_1 ("butlast", butlast);
@@ -4051,7 +4169,6 @@ siod_init (int argc, char **argv, int warnflag, FILE *fp)
   freelist = NIL;
   errjmp_ok = 0;
   nointerrupt = 1;
-  interrupt_differed = 0;
   oblistvar = NIL;
   eof_val = NIL;
   sym_errobj = NIL;
