@@ -6,20 +6,23 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
- #include <sys/stat.h>
+#include <sys/stat.h>
 #include <uim/uim.h>
 #include <uim/uim-helper.h>
 #include "context.h"
 
-static int uim_fd;
+static int uim_fd = -1;
 static void (*uim_disconnect_cb)(void);
 
 static void uim_helper_client_focus(uim_context uc, int flg);
 
 #define BUFFER_SIZE 1024
-static char uim_help_buf[BUFFER_SIZE];
 /*Common buffer for some functions's temporary buffer.
   Pay attention for use.*/
+static char uim_help_buf[BUFFER_SIZE];
+static int uim_read_buf_count;
+static int uim_read_buf_error;
+static char uim_read_buf[BUFFER_SIZE];
 
 char*
 get_server_command(void)
@@ -61,7 +64,7 @@ int uim_helper_init_client_fd(void (*disconnect_cb)(void))
     return -1;
   }
   
-  if(connect(fd,&server,sizeof(server)) == -1){
+  if(connect(fd, (struct sockaddr *)&server,sizeof(server)) == -1){
     int serv_pid = 0;
     FILE *serv_r = NULL, *serv_w = NULL;
     char buf[128];
@@ -77,17 +80,30 @@ int uim_helper_init_client_fd(void (*disconnect_cb)(void))
 	break;
     }
     
-    if(connect(fd,&server,sizeof(server)) == -1){
+    if(connect(fd, (struct sockaddr *)&server,sizeof(server)) == -1){
       return -1;
     }
   }
-  
+
   uim_disconnect_cb = disconnect_cb;
-  
+  uim_read_buf_count = 0;
+  uim_read_buf_error = 0;
   uim_fd = fd;
   return fd;
 }
 
+void
+uim_helper_close_client_fd(int fd)
+{
+  if (fd != -1) {
+    close(fd);
+  }
+  if (uim_disconnect_cb) {
+    uim_disconnect_cb();
+  }
+  uim_fd = -1;
+}
+
 
 void
 uim_helper_client_focus_in(uim_context uc)
@@ -127,29 +143,50 @@ uim_helper_client_get_prop_list(void)
   uim_helper_send_message(uim_fd, uim_help_buf);
 }
 
-
-char *
+void
 uim_helper_read_proc(int fd)
 {
-  int i,rc;  
-  char buf[1024];
-  char *tmp = strdup("");
+  int rc;  
 
-  while((i = uim_helpler_fd_readable(fd)) > 0) {
-    
-    rc = read(fd, buf, sizeof(buf)-1);
-    
-    if(rc == 0 && uim_disconnect_cb) {
+  rc = read(fd, uim_read_buf, BUFFER_SIZE - uim_read_buf_count);
+  if (rc == 0) {
+    if (uim_disconnect_cb) {
       uim_disconnect_cb();
-      uim_fd = -1;
-      return tmp;
     }
-    
-    buf[rc] = '\0';
-    tmp = (char*)realloc(tmp, strlen(tmp) + strlen(buf)+1);
-    
-    strcat(tmp, buf);
+    uim_fd = -1;
+    return ;
   }
-  
-  return tmp;
+  uim_read_buf_count += rc;
+}
+
+static void
+shift_read_buffer(int count)
+{
+  memmove(uim_read_buf, &uim_read_buf[count],
+	  uim_read_buf_count - count);
+  uim_read_buf_count -= count;
+}
+
+char *
+uim_helper_get_message()
+{
+  int i;
+  char buf[BUFFER_SIZE];
+  for (i = 0; i < uim_read_buf_count - 1; i++) {
+    if (uim_read_buf[i] == '\n' &&
+	uim_read_buf[i+1] == '\n') {
+      memcpy(buf, uim_read_buf, i+2);
+      buf[i+2] = 0;
+      shift_read_buffer(i+2);
+      if (uim_read_buf_error) {
+	uim_read_buf_error = 0;
+	return NULL;
+      }
+      return strdup(buf);
+    }
+  }
+  if (uim_read_buf_count) {
+    uim_read_buf_error = 1;
+  }
+  return NULL;
 }
