@@ -111,7 +111,7 @@ uim_ipc_open_command(int old_pid, FILE **read_handler,
 
   if (new_pid == 0) {
     /* child */
-    result = execlp(command, NULL);
+    result = execlp(command, command, NULL);
     
     if(result == -1) {
       write(1,"err",strlen("err"));
