@@ -4,8 +4,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <uim/uim.h>
 #include "config.h"
+#include <uim/uim.h>
 #include "context.h"
 #include "gettext.h"
 
@@ -13,6 +13,10 @@
 extern long siod_verbose_level;
 extern char *siod_lib;
 extern char *uim_return_str;
+extern char *uim_return_str_list[10];
+/* duplicate definition */
+#define UIM_RETURN_STR_LIST_SIZE ((sizeof(uim_return_str_list) \
+                                   / sizeof(uim_return_str_list[0])) - 2)
 
 extern void uim_init_skk_dic();
 extern void uim_quit_skk_dic();
@@ -65,13 +69,14 @@ uim_create_context(void *ptr,
 		   char *enc,
 		   char *lang,
 		   char *engine,
+		   struct uim_code_converter *conv,
 		   void (*commit_cb)(void *ptr, char *str))
 {
   uim_context uc;
   int buf_len;
   char *buf;
 
-  if (uim_siod_fatal) {
+  if (uim_siod_fatal || !conv) {
     return NULL;
   }
 
@@ -86,6 +91,7 @@ uim_create_context(void *ptr,
     enc = "UTF-8";
   }
   uc->encoding = strdup(enc);
+  uc->conv_if = conv;
   uc->conv = NULL;
   /**/
   uc->nr_modes = 0;
@@ -106,9 +112,10 @@ uim_create_context(void *ptr,
   uc->prop_list_update_cb  = NULL;
   uc->prop_label_update_cb = NULL;
   /**/
-  uc->candidate_begin_cb = NULL;
-  uc->candidate_update_cb = NULL;
-  uc->candidate_end_cb = NULL;
+  uc->candidate_selector_begin_cb = NULL;
+  uc->candidate_selector_select_cb = NULL;
+  uc->candidate_selector_shift_page_cb = NULL;
+  uc->candidate_selector_end_cb = NULL;
   /**/
   uc->nr_candidates = 0;
   uc->candidate_index = 0;
@@ -153,7 +160,7 @@ uim_release_context(uim_context uc)
   uim_eval_string(uc, buf);
   put_context_id(uc);
   if (uc->conv) {
-    iconv_close(uc->conv);
+    uc->conv_if->release(uc->conv);
   }
   for (i = 0; i < uc->nr_modes; i++) {
     free(uc->modes[i]);
@@ -255,23 +262,6 @@ uim_prop_label_update(uim_context uc)
   uc->prop_label_update_cb(uc, uc->proplabelstr);
 }
 
-
-static int
-can_convert_p(char *from, char *to)
-{
-  iconv_t ic;
-  if (!strcmp("UTF-8", from)) {
-    return 1;
-  }
-  /* TODO cache the result */
-  ic = iconv_open(to, from);
-  if (ic == (iconv_t)-1) {
-    return 0;
-  }
-  iconv_close(ic);
-  return 1;
-}
-
 int
 uim_get_nr_im(uim_context uc)
 {
@@ -281,7 +271,7 @@ uim_get_nr_im(uim_context uc)
     return 0;
 
   for (i = 0; i < uim_nr_im; i++) {
-    if (can_convert_p(uim_im_array[i].encoding, uc->encoding)) {
+    if (uc->conv_if->is_convertible(uc->encoding, uim_im_array[i].encoding)) {
       nr ++;
     }
   }
@@ -293,7 +283,7 @@ get_nth_im(uim_context uc, int nth)
 {
   int i,n=0;
   for (i = 0; i < uim_nr_im; i++) {
-    if (can_convert_p(uim_im_array[i].encoding, uc->encoding)) {
+    if (uc->conv_if->is_convertible(uc->encoding, uim_im_array[i].encoding)) {
       if (n == nth) {
 	return &uim_im_array[i];
       }
@@ -325,6 +315,17 @@ uim_get_im_language(uim_context uc, int nth)
   return NULL;
 }
 
+const char *
+uim_get_im_encoding(uim_context uc, int nth)
+{
+  struct uim_im *im = get_nth_im(uc, nth);
+
+  if(im) {
+    return im->encoding;
+  }
+  return NULL;
+}
+
 static int
 load_conf()
 {
@@ -350,27 +351,60 @@ load_conf()
   return ret;
 }
 
-int uim_set_candidate_cb(uim_context uc,
-			 void (*begin_cb)(void *ptr, int nr, int index),
-			 void (*update_cb)(void *ptr, int index),
-			 void (*end_cb)(void *ptr))
+int uim_set_candidate_selector_cb(uim_context uc,
+				  void (*begin_cb)(void *ptr, int nr, int display_limit),
+				  void (*select_cb)(void *ptr, int index),
+				  void (*shift_page_cb)(void *ptr, int direction),
+				  void (*end_cb)(void *ptr))
 {
-  uc->candidate_begin_cb = begin_cb;
-  uc->candidate_update_cb = update_cb;
-  uc->candidate_end_cb = end_cb;
+  uc->candidate_selector_begin_cb = begin_cb;
+  uc->candidate_selector_select_cb = select_cb;
+  uc->candidate_selector_end_cb = end_cb;
+  uc->candidate_selector_shift_page_cb = shift_page_cb;
+
   return 0;
 }
 
-char *uim_get_candidate(uim_context uc, int nth)
+uim_candidate
+uim_get_candidate(uim_context uc, int index, int accel_enumeration_hint)
 {
+  int i;
   char buf[100];
   char *tmp, *str;
-  snprintf(buf, 99, "(get-candidate %d %d)", uc->id, nth);
+  uim_candidate cand = malloc(sizeof(*cand));
+  snprintf(buf, 99, "(get-candidate %d %d %d)", uc->id, index, accel_enumeration_hint);
   uim_eval_string(uc, buf);
-  if (uim_return_str) {
-    return uim_code_conv(uim_return_str, uc->conv);
+
+  if (uim_return_str_list[0] && uim_return_str_list[1]) {
+    cand->str = uc->conv_if->convert(uc->conv, uim_return_str_list[0]);
+    cand->heading_label = uc->conv_if->convert(uc->conv, uim_return_str_list[1]);    
+  } else {
+    cand->str = NULL;
+    cand->heading_label = NULL;
   }
-  return NULL;
+
+  return cand;
+}
+
+const char *
+uim_candidate_get_cand_str(uim_candidate cand)
+{
+  return cand->str;
+}
+
+const char *
+uim_candidate_get_heading_label(uim_candidate cand)
+{
+  return cand->heading_label;
+}
+
+void
+uim_candidate_free(uim_candidate cand)
+{
+  free(cand->str);
+  free(cand->heading_label);
+  /* free(cand->annotation); */
+  free(cand);
 }
 
 int uim_get_candidate_index(uim_context uc)
@@ -397,11 +431,12 @@ exit_hook(void)
 static void
 uim_init_scm()
 {
+  int i;
   char *scm_files;
   char *siod_argv[] =
     {
       "siod",
-      "-h100000:10",
+      "-h300000:10",
       "-o1000",
       "-s200000",
       "-n128",
@@ -425,6 +460,7 @@ uim_init_scm()
   /**/
   uim_init_im_subrs();
   uim_init_util_subrs();
+  uim_init_key_subrs();
   uim_init_table_subrs();
   uim_init_anthy();
   uim_init_prime();
@@ -449,6 +485,11 @@ uim_init_scm()
   if (getenv("LIBUIM_VERBOSE")) {
     siod_verbose_level = 10;
   }
+
+  uim_return_str = NULL;
+  for (i = 0; i < UIM_RETURN_STR_LIST_SIZE; i++){
+    uim_return_str_list[i] = NULL;
+  }
 }
 
 int
