@@ -14,15 +14,20 @@ static int candwin_pid = 0;
 static char*
 candwin_command(void)
 {
+#ifdef USE_GTK2
   return "uim-helper-candwin-gtk";
+#else
+  return NULL;
+#endif
 }
 
 Canddisp* canddisp_singleton()
 {
     static Canddisp* disp = NULL;
+    char *command = candwin_command();
 
-    if (disp == NULL) {
-      candwin_pid = uim_ipc_open_command(candwin_pid, &candwin_r, &candwin_w, candwin_command() );
+    if (disp == NULL && command) {
+      candwin_pid = uim_ipc_open_command(candwin_pid, &candwin_r, &candwin_w, command);
       disp = new Canddisp();
     }
     return disp;
@@ -51,11 +56,11 @@ void Canddisp::begin(std::vector<const char*> candidates)
   fflush(candwin_w);
 }
 
-void Canddisp::update(int index)
+void Canddisp::select(int index)
 {
   if(candwin_w == NULL)
     return;
-  fprintf(candwin_w, "update\n");
+  fprintf(candwin_w, "select\n");
   fprintf(candwin_w, "%d\n\n", index);
   fflush(candwin_w);
 }
