@@ -9,7 +9,7 @@ public:
     Canddisp();
     virtual ~Canddisp();
     void begin(std::vector<const char*>);
-    void update(int index);
+    void select(int index);
     void end();
     void show();
     void hide();
