@@ -11,12 +11,10 @@
 #include "xdispatch.h"
 
 //
-// PeWinץꥨǥåѤδܥɥ
+// PeWin: Base class for preedit window
 //  PeLineWin  root window style
 //  PeOvWin  over the spot
 
-//IMEϥ󥿥饯ƥ֤ǤɬפΤǡ¤XFlushȤ
-
 static XFontSet gFontset;
 
 void init_convdisp()
@@ -29,7 +27,7 @@ void init_convdisp()
 		       "-misc-fixed-medium-r-normal--14-130-75-75-c-140-jisx0208.1983-0,"
 		       "-misc-fixed-medium-r-normal--14-130-75-75-c-70-jisx0201.1976-0",
 		       &missing, &nr_missing, &def);
-    // ktermλȤեȥåȤäƤ
+    // same fontset as kterm
 
     if (missing) {
 	XFreeStringList(missing);
@@ -44,7 +42,7 @@ struct char_ent {
     int x, y;
 };
 
-// ץꥨǥåWindow
+// Preedit Window
 class PeWin : public WindowIf {
 public:
     PeWin(Window pw);
@@ -80,7 +78,7 @@ protected:
     bool mIsMapped;
 };
 
-// RootWindowStyleѤ1ԥɥ
+// one line preedit window for RootWindowStyle
 class PeLineWin : public PeWin {
 public:
     PeLineWin(Window w);
@@ -96,7 +94,7 @@ private:
     int m_x;
 };
 
-// over the spotѤΥɥ
+// window for over the spot style
 class PeOvWin : public PeWin {
 public:
     PeOvWin(Window w);
@@ -134,7 +132,7 @@ private:
     PeOvWin *m_ov_win;
 };
 
-// RootWindow
+// Preedit window for RootWindowStyle
 class ConvdispRw : public Convdisp{
 public:
     ConvdispRw(InputContext *,icxatr *);
@@ -182,13 +180,13 @@ Convdisp *create_convdisp(int style, InputContext *k,
 }
 
 //
-// PeWin(PreeEdit Window)
+// PeWin(PreEdit Window)
 //
 PeWin::PeWin(Window pw)
 {
     mParentWin = pw;
     int scr_num = DefaultScreen(XimServer::gDpy);
-    // Ȥꤢƥȡ
+    // tentative
     mWidth = 1;
     mHeight = 1;
     mWin = XCreateSimpleWindow(XimServer::gDpy, mParentWin,
@@ -213,7 +211,7 @@ PeWin::PeWin(Window pw)
     XSetBackground(XimServer::gDpy, mClearGC, BlackPixel(XimServer::gDpy, scr_num));
 
     add_window_watch(mWin, this, EXPOSE_MASK|STRUCTURE_NOTIFY_MASK);
-    mIsMapped = false;//ޥåפƤʤ
+    mIsMapped = false;//not mapped now
     mFontset = gFontset;
     
     XFlush(XimServer::gDpy);
@@ -482,7 +480,6 @@ void Convdisp::set_pe(pe_stat *p)
     m_pe = p;
 }
 
-//롼ȥɥ
 ConvdispRw::ConvdispRw(InputContext *k, icxatr *a) : Convdisp(k, a)
 {
     mPeWin = new PeLineWin(DefaultRootWindow(XimServer::gDpy));
@@ -496,7 +493,7 @@ ConvdispRw::~ConvdispRw()
 void ConvdispRw::update_preedit()
 {
     if (m_pe && m_pe->get_char_count()) {
-	// preeditȤ¸ߤ
+	// preedit string exists
 	mPeWin->do_map();
 	mPeWin->draw_pe(m_pe);
 	mPeWin->draw();
@@ -536,22 +533,35 @@ void ConvdispOv::update_preedit()
 	return ;
     }
 
-    // preedit
     draw_preedit();
 
-    if (m_atr->has_atr(ICA_FocusWindow)
-        && m_atr->has_atr(ICA_SpotLocation)){
-	int x, y;
+    if (m_atr->has_atr(ICA_SpotLocation)){
+       
+	int x = -1, y = -1;
 	Window win;
-	XTranslateCoordinates(XimServer::gDpy,m_atr->focus_window,
-			      DefaultRootWindow(XimServer::gDpy),
-			      m_atr->spot_location.x,
-			      m_atr->spot_location.y,
-			      &x, &y, &win);
-        
-        Canddisp* disp = canddisp_singleton();
-        disp->move(x, y + 2);
-    }    
+
+	if(m_atr->has_atr(ICA_FocusWindow)){
+
+	    XTranslateCoordinates(XimServer::gDpy,m_atr->focus_window,
+				  DefaultRootWindow(XimServer::gDpy),
+				  m_atr->spot_location.x,
+				  m_atr->spot_location.y,
+				  &x, &y, &win);
+	}
+      
+	if(m_atr->has_atr(ICA_ClientWindow)){
+
+	    XTranslateCoordinates(XimServer::gDpy,m_atr->client_window,
+				  DefaultRootWindow(XimServer::gDpy),
+				  m_atr->spot_location.x,
+				  m_atr->spot_location.y,
+				  &x, &y, &win);
+	}
+	if(x > -1 && y > -1) {
+	    Canddisp* disp = canddisp_singleton();
+	    disp->move(x, y + 2);
+	}
+    }
 }
 
 void ConvdispOv::validate_area()
@@ -578,7 +588,7 @@ void ConvdispOv::update_icxatr()
     }
     
     if (m_atr->is_changed(ICA_FocusWindow)) {
-	//ʤ˺줿ɡFocusWindow夫ꤷƤΤ
+	//Some clients send FocusWindow later
         if (!check_win()) {
             return ;
         }
@@ -628,9 +638,9 @@ void ConvdispOv::draw_preedit()
     }
 
     m_ce = (char_ent *)malloc(sizeof(char_ent)*m_ce_len);
-    make_ce_array();//ơ
-    layoutCharEnt();//쥤Ȥơ
-    do_draw_preedit();//
+    make_ce_array();//prepare array
+    layoutCharEnt();//layout them
+    do_draw_preedit();//do draw
     free(m_ce);
     m_ov_win->draw();
     XFlush(XimServer::gDpy);
@@ -638,19 +648,23 @@ void ConvdispOv::draw_preedit()
 
 void ConvdispOv::do_draw_preedit()
 {
-    m_ov_win->set_pos(0, 0);
+    if (m_atr->has_atr(ICA_Area)) {
+	m_ov_win->set_pos(m_atr->area.x, m_atr->area.y);
+    } else {
+	m_ov_win->set_pos(0, 0);
+    }
     m_ov_win->draw_ce(m_ce, m_ce_len);
 }
 
 bool ConvdispOv::check_win()
 {
     if (!check_atr()) {
-	//ץꥨǥåȥɥФΤ˽ʬʾ̵
+	//not enough information to map preedit
 	return false;
     }
   
     if (m_ov_win && !m_atr->is_changed(ICA_FocusWindow)) {
-	// ɥupdateɬפ̵
+	// no need to update window
 	return true;
     }
 
@@ -762,6 +776,12 @@ void ConvdispOv::layoutCharEnt()
 	m_ce[i].y = y;
 	x += m_ce[i].width;
     }
+    if (m_atr->has_atr(ICA_Area)) {
+	for (i = 0; i < m_ce_len; i++) {
+	    m_ce[i].x -= m_atr->area.x;
+	    m_ce[i].y -= m_atr->area.y;
+	}
+    }
 }
 
 int ConvdispOv::calc_ce_width(int b, int e)
@@ -814,7 +834,7 @@ void ConvdispOs::update_preedit()
     t->pushC32(caret_pos);// caret
     t->pushC32(0); // chg_first
     t->pushC32(mPrevLen); // chg_length
-    //feedback array 򥻥åȤʤmozilla뤬IMEȹͤΤ̤
+
     if (m_pe->ustrings.size()) {
 	t->pushC32(0);
     } else {
