@@ -381,9 +381,10 @@ main(int argc, char **argv)
     int res;
     printf("UIM-XIM bridge. Now only supports Japanese.\n");
 
-    setlocale(LC_ALL, "");
     get_runtime_env();
     parse_args(argc, argv);
+    setlocale(LC_ALL, "ja_JP.eucJP");
+    signal(SIGPIPE , SIG_IGN);
 
     lib_uim_fd = uim_helper_init_client_fd(helper_disconnect_cb);
     if (lib_uim_fd >= 0) {
@@ -405,7 +406,7 @@ main(int argc, char **argv)
 	return 1;
     }
     uim_context uc = uim_create_context(NULL, "EUC-JP", NULL,
-					NULL, NULL);
+					NULL, uim_iconv, NULL);
     int nr = uim_get_nr_im(uc);
     for (int i = 0; i < nr; i++) {
 	char *name = (char *)uim_get_im_name(uc, i);
