@@ -55,8 +55,8 @@ get_font_set(char *name)
     struct FSCache fc;
     char **missing,*def;
     int nr_missing;
-    fc.fs = XCreateFontSet(XimServer::gDpy,name,
-			   &missing,&nr_missing,&def);
+    fc.fs = XCreateFontSet(XimServer::gDpy, name,
+			   &missing, &nr_missing, &def);
     if (missing) {
 	XFreeStringList(missing);
     }
@@ -275,6 +275,8 @@ int icxatr::getSize(int id)
 	return 4;
     case ICA_FilterEvents:
 	return 4;
+    case ICA_InputStyle:
+	return 4;
     }
     return 0;
 }
@@ -463,6 +465,9 @@ int XimIC::get_ic_atr(int id, TxPacket *t)
     case ICA_FilterEvents:
 	t->pushC32(KeyPressMask);
 	break;
+    case ICA_InputStyle:
+	t->pushC32(m_xatr.input_style);
+  	break;
     default:
 	printf("try to get unknown ic attribute %d.\n",id);
 	break;
