@@ -125,8 +125,8 @@ public:
     void clear_preedit();
     void pushback_preedit_string(int attr, char *str);
     void update_preedit();
-    void candidate_begin(int nr, int index);
-    void candidate_update(int index);
+    void candidate_begin(int nr, int display_limit);
+    void candidate_select(int index);
     void candidate_end();
     void update_prop_list(char *str);
     void update_prop_label(char *str);
@@ -136,7 +136,7 @@ public:
     static void pushback_cb(void *, int attr, char *str);
     static void update_cb(void *);
     static void candidate_begin_cb(void *ptr, int nr, int index);
-    static void candidate_update_cb(void *ptr, int index);
+    static void candidate_select_cb(void *ptr, int index);
     static void candidate_end_cb(void *ptr);
     static void update_prop_list_cb(void *ptr, char *str);
     static void update_prop_label_cb(void *ptr, char *str);
